@@ -75,6 +75,7 @@ import org.apache.cxf.endpoint.ServerRegistry;
 import org.apache.cxf.helpers.IOUtils;
 import org.apache.cxf.helpers.LoadingByteArrayOutputStream;
 import org.apache.cxf.helpers.XMLUtils;
+import org.apache.cxf.jaxb.JAXBContextCache;
 import org.apache.cxf.resource.ExtendedURIResolver;
 import org.apache.cxf.resource.ResourceManager;
 import org.apache.cxf.service.model.SchemaInfo;
@@ -195,17 +196,19 @@ public final class EndpointReferenceUtils {
             // handle case where given systemId is null (so that
             // direct key lookup fails) by scanning through map
             // searching for a namespace match
-            for (Map.Entry<String, byte[]> ent : schemas.entrySet()) {
-                if (ent.getKey().endsWith(namespaceURI)) {
-                    schemas.remove(ent.getKey());
-                    impl = new LSInputImpl();
-                    impl.setSystemId(newId);
-                    impl.setBaseURI(newId);
-                    impl.setCharacterStream(
-                        new InputStreamReader(
-                            new ByteArrayInputStream(ent.getValue())));
-                    done.add(newId + ":" + namespaceURI);
-                    return impl;
+            if (namespaceURI != null) {
+                for (Map.Entry<String, byte[]> ent : schemas.entrySet()) {
+                    if (ent.getKey().endsWith(namespaceURI)) {
+                        schemas.remove(ent.getKey());
+                        impl = new LSInputImpl();
+                        impl.setSystemId(newId);
+                        impl.setBaseURI(newId);
+                        impl.setCharacterStream(
+                            new InputStreamReader(
+                                new ByteArrayInputStream(ent.getValue())));
+                        done.add(newId + ":" + namespaceURI);
+                        return impl;
+                    }
                 }
             }
                 
@@ -227,8 +230,8 @@ public final class EndpointReferenceUtils {
         }
     }
 
-    public static final String ANONYMOUS_ADDRESS = "http://www.w3.org/2005/08/addressing/anonymous";
-
+    public static final String ANONYMOUS_ADDRESS = WSAEndpointReferenceUtils.ANONYMOUS_ADDRESS;
+    
     private static final Logger LOG = LogUtils.getL7dLogger(EndpointReferenceUtils.class);
 
     private static final String NS_WSAW_2005 = "http://www.w3.org/2005/02/addressing/wsdl";
@@ -255,9 +258,13 @@ public final class EndpointReferenceUtils {
     
     private static final org.apache.cxf.ws.addressing.wsdl.ObjectFactory WSA_WSDL_OBJECT_FACTORY = 
         new org.apache.cxf.ws.addressing.wsdl.ObjectFactory();
-    private static final org.apache.cxf.ws.addressing.ObjectFactory WSA_OBJECT_FACTORY = 
-        new org.apache.cxf.ws.addressing.ObjectFactory();
-
+    
+    
+    private static final Set<Class<?>> ADDRESSING_CLASSES = new HashSet<Class<?>>();
+    static {
+        ADDRESSING_CLASSES.add(WSA_WSDL_OBJECT_FACTORY.getClass());
+        ADDRESSING_CLASSES.add(WSAEndpointReferenceUtils.WSA_OBJECT_FACTORY.getClass());
+    }
     
     private EndpointReferenceUtils() {
         // Utility class - never constructed
@@ -274,11 +281,7 @@ public final class EndpointReferenceUtils {
                                              String portName) {
         if (null != serviceName) {
             JAXBElement<ServiceNameType> jaxbElement = getServiceNameType(serviceName, portName);
-            MetadataType mt = ref.getMetadata();
-            if (null == mt) {
-                mt = new MetadataType();
-                ref.setMetadata(mt);
-            }
+            MetadataType mt = WSAEndpointReferenceUtils.getSetMetadata(ref);
 
             mt.getAny().add(jaxbElement);
         }
@@ -415,11 +418,7 @@ public final class EndpointReferenceUtils {
             JAXBElement<AttributedQNameType> jaxbElement = 
                 WSA_WSDL_OBJECT_FACTORY.createInterfaceName(interfaceNameType);
 
-            MetadataType mt = ref.getMetadata();
-            if (null == mt) {
-                mt = WSA_OBJECT_FACTORY.createMetadataType();
-                ref.setMetadata(mt);
-            }
+            MetadataType mt = WSAEndpointReferenceUtils.getSetMetadata(ref);
             mt.getAny().add(jaxbElement);
         }
     }
@@ -493,11 +492,7 @@ public final class EndpointReferenceUtils {
 
     public static void setWSDLLocation(EndpointReferenceType ref, String... wsdlLocation) {
         
-        MetadataType metadata = ref.getMetadata();
-        if (null == metadata) {
-            metadata = WSA_OBJECT_FACTORY.createMetadataType();
-            ref.setMetadata(metadata);
-        }
+        MetadataType metadata = WSAEndpointReferenceUtils.getSetMetadata(ref);
 
         //wsdlLocation attribute is a list of anyURI.
         StringBuilder strBuf = new StringBuilder();
@@ -539,11 +534,7 @@ public final class EndpointReferenceUtils {
         throws EndpointUtilsException {
         
         if (null != ref) {
-            MetadataType mt = ref.getMetadata();
-            if (null == mt) {
-                mt = new MetadataType();
-                ref.setMetadata(mt);
-            }
+            MetadataType mt = WSAEndpointReferenceUtils.getSetMetadata(ref);
             List<Object> anyList = mt.getAny();
             try {
                 for (Source source : metadata) {
@@ -629,7 +620,7 @@ public final class EndpointReferenceUtils {
     }
     
     
-    private static Schema createSchema(ServiceInfo serviceInfo, Bus b) {
+    private static synchronized Schema createSchema(ServiceInfo serviceInfo, Bus b) {
         if (b == null) {
             b = BusFactory.getThreadDefaultBus(false);
         }
@@ -642,6 +633,7 @@ public final class EndpointReferenceUtils {
             try {
                 for (SchemaInfo si : serviceInfo.getSchemas()) {
                     Element el = si.getElement();
+                    unsetReadonly(el);
                     String baseURI = null;
                     try {
                         baseURI = el.getBaseURI();
@@ -704,12 +696,29 @@ public final class EndpointReferenceUtils {
                     String s = XMLUtils.toString(schemaInfo.getElement(), 4);
                     LOG.log(Level.INFO, "Schema for: " + schemaInfo.getNamespaceURI() + "\n" + s);
                 }
+            } finally { 
+                for (Source src : schemaSourcesMap2.values()) {
+                    if (src instanceof DOMSource) {
+                        Node nd = ((DOMSource)src).getNode();
+                        unsetReadonly(nd);
+                    }
+                }
             }
             serviceInfo.setProperty(Schema.class.getName(), schema);
         }
         return schema;
     }
     
+    private static void unsetReadonly(Node nd) {
+        try {
+            //work around a bug in the version of Xerces that is in the JDK
+            //that only allows the Element to be used to create a schema once.
+            nd.getClass().getMethod("setReadOnly", Boolean.TYPE, Boolean.TYPE).invoke(nd, false, true);
+        } catch (Throwable ex) {
+            //ignore
+        }
+    }
+
     public static Schema getSchema(ServiceInfo serviceInfo) {
         return getSchema(serviceInfo, null);
     }
@@ -817,13 +826,7 @@ public final class EndpointReferenceUtils {
      * @return String the address of the endpoint
      */
     public static String getAddress(EndpointReferenceType ref) {
-        AttributedURIType a = ref.getAddress();
-        
-        if (null != a) {
-            return a.getValue();
-        }
-        // should wsdl be parsed for an address now?
-        return null;
+        return WSAEndpointReferenceUtils.getAddress(ref);
     }
 
     /**
@@ -832,9 +835,7 @@ public final class EndpointReferenceUtils {
      * @param address - the address
      */
     public static void setAddress(EndpointReferenceType ref, String address) {
-        AttributedURIType a = WSA_OBJECT_FACTORY.createAttributedURIType();
-        a.setValue(address);
-        ref.setAddress(a);
+        WSAEndpointReferenceUtils.setAddress(ref, address);
     }
     /**
      * Create an endpoint reference for the provided wsdl, service and portname.
@@ -846,8 +847,8 @@ public final class EndpointReferenceUtils {
     public static EndpointReferenceType getEndpointReference(URL wsdlUrl, 
                                                              QName serviceName,
                                                              String portName) {
-        EndpointReferenceType reference = WSA_OBJECT_FACTORY.createEndpointReferenceType();
-        reference.setMetadata(WSA_OBJECT_FACTORY.createMetadataType());
+        EndpointReferenceType reference = 
+            WSAEndpointReferenceUtils.createEndpointReferenceWithMetadata();
         setServiceAndPortName(reference, serviceName, portName);
         //TODO To Ensure it is a valid URI syntax.
         setWSDLLocation(reference, wsdlUrl.toString());
@@ -863,11 +864,7 @@ public final class EndpointReferenceUtils {
      */
     public static EndpointReferenceType duplicate(EndpointReferenceType ref) {
 
-        EndpointReferenceType reference = WSA_OBJECT_FACTORY.createEndpointReferenceType();
-        reference.setMetadata(ref.getMetadata());
-        reference.getAny().addAll(ref.getAny());
-        reference.setAddress(ref.getAddress());
-        return reference;
+        return WSAEndpointReferenceUtils.duplicate(ref);
     }
     
     /**
@@ -877,16 +874,12 @@ public final class EndpointReferenceUtils {
      */
     public static EndpointReferenceType getEndpointReference(String address) {
 
-        EndpointReferenceType reference = WSA_OBJECT_FACTORY.createEndpointReferenceType();
-        setAddress(reference, address);
-        return reference;
+        return WSAEndpointReferenceUtils.getEndpointReference(address);
     }
     
     public static EndpointReferenceType getEndpointReference(AttributedURIType address) {
 
-        EndpointReferenceType reference = WSA_OBJECT_FACTORY.createEndpointReferenceType();
-        reference.setAddress(address);
-        return reference;
+        return WSAEndpointReferenceUtils.getEndpointReference(address);
     }    
     
     /**
@@ -895,9 +888,7 @@ public final class EndpointReferenceUtils {
      */
     public static EndpointReferenceType getAnonymousEndpointReference() {
         
-        EndpointReferenceType reference = WSA_OBJECT_FACTORY.createEndpointReferenceType();
-        setAddress(reference, ANONYMOUS_ADDRESS);
-        return reference;
+        return WSAEndpointReferenceUtils.getAnonymousEndpointReference();
     }
     
     /**
@@ -1024,9 +1015,9 @@ public final class EndpointReferenceUtils {
     
     public static Source convertToXML(EndpointReferenceType epr) {
         try {
-            JAXBContext jaxbContext = 
-                JAXBContext.newInstance(new Class[] {WSA_WSDL_OBJECT_FACTORY.getClass(), 
-                                                     WSA_OBJECT_FACTORY.getClass()});
+            JAXBContext jaxbContext = JAXBContextCache.getCachedContextAndSchemas(ADDRESSING_CLASSES,
+                                                                                  null, null, null,
+                                                                                  false).getContext();
             javax.xml.bind.Marshaller jm = jaxbContext.createMarshaller();
             jm.setProperty(Marshaller.JAXB_FRAGMENT, true);
             QName qname = new QName("http://www.w3.org/2005/08/addressing", "EndpointReference");
