@@ -62,10 +62,10 @@ import javax.xml.namespace.QName;
 import org.w3c.dom.Element;
 import org.w3c.dom.NamedNodeMap;
 import org.w3c.dom.Node;
-
 import org.xml.sax.InputSource;
 import org.xml.sax.SAXParseException;
 
+
 import org.apache.cxf.Bus;
 import org.apache.cxf.catalog.OASISCatalogManager;
 import org.apache.cxf.catalog.OASISCatalogManagerHelper;
@@ -133,6 +133,7 @@ public class SourceGenerator {
     private List<String> generatedTypeClasses = new ArrayList<String>();
     private List<InputSource> bindingFiles = Collections.emptyList();
     private List<InputSource> schemaPackageFiles = Collections.emptyList();
+    private List<String> compilerArgs = new ArrayList<String>();
     private Map<String, String> schemaPackageMap = Collections.emptyMap();
     private Bus bus;
     
@@ -576,9 +577,29 @@ public class SourceGenerator {
     private boolean writeResponseType(List<Element> responseEls, StringBuilder sbCode,
                                    Set<String> imports, Set<String> typeClassNames, 
                                    GrammarInfo gInfo) {
-        List<Element> repElements = responseEls.size() == 1 
-            ? DOMUtils.getChildrenWithName(responseEls.get(0), WadlGenerator.WADL_NS, "representation")
-            : CastUtils.cast(Collections.emptyList(), Element.class);
+        List<Element> repElements = null;
+        if (responseEls.size() >= 1) {
+            Element okResponse = null;
+            if (responseEls.size() > 1) {
+                for (int i = 0; i < responseEls.size(); i++) {
+                    String statusValue = responseEls.get(i).getAttribute("status");
+                    try {
+                        int status = statusValue.length() == 0 ? 200 : Integer.valueOf(statusValue);
+                        if (status == 200) {
+                            okResponse = responseEls.get(i);
+                            break;
+                        }
+                    } catch (NumberFormatException ex) {
+                        // ignore
+                    }
+                }
+            } else {
+                okResponse = responseEls.get(0);
+            }
+            repElements = DOMUtils.getChildrenWithName(okResponse, WadlGenerator.WADL_NS, "representation");
+        } else {
+            repElements = CastUtils.cast(Collections.emptyList(), Element.class);
+        }
         if (repElements.size() == 0) {    
             sbCode.append("void ");
             return false;
@@ -718,7 +739,7 @@ public class SourceGenerator {
         if (localName == null) {
             return null;
         }
-        String clsName = packageName + "." + localName;
+        String clsName = packageName + "." + localName.toLowerCase();
         for (String type : typeClassNames) {
             if (type.toLowerCase().equals(clsName)) {
                 return type;
@@ -816,8 +837,12 @@ public class SourceGenerator {
         List<SchemaInfo> schemas = new ArrayList<SchemaInfo>();
         List<Element> schemasEls = DOMUtils.getChildrenWithName(grammarEls.get(0), 
              XmlSchemaConstants.XSD_NAMESPACE_URI, "schema");
-        for (Element schemaEl : schemasEls) {
-            schemas.add(createSchemaInfo(schemaEl, app.getWadlPath()));
+        for (int i = 0; i < schemasEls.size(); i++) {
+            String systemId = app.getWadlPath();
+            if (schemasEls.size() > 1) {
+                systemId += "#grammar" + (i + 1);
+            }
+            schemas.add(createSchemaInfo(schemasEls.get(i), systemId));
         }
         List<Element> includeEls = DOMUtils.getChildrenWithName(grammarEls.get(0), 
              WadlGenerator.WADL_NS, "include");
@@ -880,6 +905,10 @@ public class SourceGenerator {
         
 
         SchemaCompiler compiler = createCompiler(type);
+        if (compilerArgs.size() > 0) {
+            compiler.getOptions().addGrammar(new InputSource("null"));
+            compiler.getOptions().parseArguments(compilerArgs.toArray(new String[] {}));
+        }
         addSchemas(schemaElements, compiler);
         for (InputSource is : bindingFiles) {
             compiler.getOptions().addBindFile(is);
@@ -971,6 +1000,10 @@ public class SourceGenerator {
     public void setSchemaPackageFiles(List<InputSource> files) {
         this.schemaPackageFiles = files;
     }
+    
+    public void setCompilerArgs(List<String> args) {
+        this.compilerArgs = args;
+    }
 
     public void setSchemaPackageMap(Map<String, String> map) {
         this.schemaPackageMap = map;
