@@ -70,6 +70,10 @@ public class JSONProvider extends AbstractJAXBProvider  {
     private static final String MAPPED_CONVENTION = "mapped";
     private static final String BADGER_FISH_CONVENTION = "badgerfish";
     
+    static {
+        new SimpleConverter();
+    }
+    
     private ConcurrentHashMap<String, String> namespaceMap = 
         new ConcurrentHashMap<String, String>();
     private boolean serializeAsArray;
@@ -85,6 +89,12 @@ public class JSONProvider extends AbstractJAXBProvider  {
     private boolean ignoreNamespaces;
     private String convention = MAPPED_CONVENTION;
     private TypeConverter typeConverter;
+    private boolean attributesToElements;
+    
+    @Override
+    public void setAttributesToElements(boolean value) {
+        this.attributesToElements = value;
+    }
     
     public void setConvention(String value) {
         if (!MAPPED_CONVENTION.equals(value) && !BADGER_FISH_CONVENTION.equals(value)) {
@@ -174,8 +184,8 @@ public class JSONProvider extends AbstractJAXBProvider  {
         
         try {
             boolean isCollection = InjectionUtils.isSupportedCollectionOrArray(type);
-            Class<?> theType = isCollection ? InjectionUtils.getActualType(genericType) : type;
-            theType = getActualType(theType, genericType, anns);
+            Class<?> theGenericType = isCollection ? InjectionUtils.getActualType(genericType) : type;
+            Class<?> theType = getActualType(theGenericType, genericType, anns);
             
             Unmarshaller unmarshaller = createUnmarshaller(theType, genericType, isCollection);
             
@@ -195,7 +205,7 @@ public class JSONProvider extends AbstractJAXBProvider  {
             }
             if (isCollection) {
                 response = ((CollectionWrapper)response).getCollectionOrArray(theType, type, 
-                                                         getAdapter(theType, anns)); 
+                               org.apache.cxf.jaxrs.utils.JAXBUtils.getAdapter(theGenericType, anns)); 
             } else {
                 response = checkAdapter(response, type, anns, false);
             }
@@ -284,6 +294,15 @@ public class JSONProvider extends AbstractJAXBProvider  {
     public void writeTo(Object obj, Class<?> cls, Type genericType, Annotation[] anns,  
         MediaType m, MultivaluedMap<String, Object> headers, OutputStream os)
         throws IOException {
+        if (os == null) {
+            StringBuilder sb = new StringBuilder();
+            sb.append("Jettison needs initialized OutputStream");
+            if (getContext() != null && getContext().getContent(XMLStreamWriter.class) == null) {
+                sb.append("; if you need to customize Jettison output with the custom XMLStreamWriter"
+                          + " then extend JSONProvider or when possible configure it directly.");
+            }
+            throw new IOException(sb.toString());
+        }
         try {
             
             String encoding = HttpUtils.getSetEncoding(m, headers, "UTF-8");
@@ -351,7 +370,8 @@ public class JSONProvider extends AbstractJAXBProvider  {
         
         os.write(startTag.getBytes());
         if (firstObj != null) {
-            XmlJavaTypeAdapter adapter = getAdapter(firstObj.getClass(), anns);
+            XmlJavaTypeAdapter adapter = 
+                org.apache.cxf.jaxrs.utils.JAXBUtils.getAdapter(firstObj.getClass(), anns);
             marshalCollectionMember(JAXBUtils.useAdapter(firstObj, adapter, true),
                                     actualClass, genericType, encoding, os);
             while (it.hasNext()) {
@@ -398,7 +418,10 @@ public class JSONProvider extends AbstractJAXBProvider  {
         QName qname = getQName(actualClass, genericType, actualObject, true);
         
         Configuration config = 
-            JSONUtils.createConfiguration(namespaceMap, writeXsiType && !ignoreNamespaces, typeConverter);
+            JSONUtils.createConfiguration(namespaceMap, 
+                                          writeXsiType && !ignoreNamespaces,
+                                          attributesToElements,
+                                          typeConverter);
         
         XMLStreamWriter writer = JSONUtils.createStreamWriter(os, qname, 
              writeXsiType && !ignoreNamespaces, config, serializeAsArray, arrayKeys,
