@@ -41,7 +41,6 @@ import java.util.TreeSet;
 
 import javax.ws.rs.GET;
 import javax.ws.rs.Path;
-import javax.ws.rs.WebApplicationException;
 import javax.ws.rs.core.MediaType;
 import javax.ws.rs.core.Response;
 import javax.xml.bind.JAXBContext;
@@ -81,7 +80,6 @@ import org.apache.cxf.jaxrs.resources.SuperBook;
 import org.apache.cxf.jaxrs.resources.TagVO;
 import org.apache.cxf.jaxrs.resources.TagVO2;
 import org.apache.cxf.jaxrs.resources.Tags;
-import org.apache.cxf.jaxrs.utils.ParameterizedCollectionType;
 import org.apache.cxf.jaxrs.utils.ResourceUtils;
 
 import org.junit.Assert;
@@ -324,33 +322,6 @@ public class JAXBElementProviderTest extends Assert {
         readSuperBook(bos.toString(), false);
     }
     
-    @Test
-    public void testWriteWithCustomPrefixes() throws Exception {
-        JAXBElementProvider provider = new JAXBElementProvider();
-        provider.setNamespacePrefixes(
-            Collections.singletonMap("http://tags", "prefix"));
-        TagVO2 tag = new TagVO2("a", "b");
-        
-        ByteArrayOutputStream bos = new ByteArrayOutputStream();
-        provider.writeTo(tag, TagVO2.class, TagVO2.class,
-                       new Annotation[0], MediaType.TEXT_XML_TYPE, new MetadataMap<String, Object>(), bos);
-        assertTrue(bos.toString().contains("prefix:thetag"));
-    }
-    
-    @Test
-    public void testWriteCollectionWithCustomPrefixes() throws Exception {
-        JAXBElementProvider provider = new JAXBElementProvider();
-        provider.setNamespacePrefixes(
-            Collections.singletonMap("http://tags", "prefix"));
-        TagVO2 tag = new TagVO2("a", "b");
-        List<TagVO2> tags = Collections.singletonList(tag);
-        ByteArrayOutputStream bos = new ByteArrayOutputStream();
-        provider.writeTo(tags, List.class, new ParameterizedCollectionType<TagVO2>(TagVO2.class),
-                       new Annotation[0], MediaType.TEXT_XML_TYPE, new MetadataMap<String, Object>(), bos);
-        assertTrue(bos.toString().contains("prefix:thetag"));
-        assertFalse(bos.toString().contains("ns1:thetag"));
-    }
-    
     @Test
     public void testWriteWithoutXmlRootElement() throws Exception {
         doTestWriteWithoutXmlRootElement("SuperBook", false, false);
@@ -411,8 +382,7 @@ public class JAXBElementProviderTest extends Assert {
                          org.apache.cxf.jaxrs.fortest.jaxb.SuperBook.class,
                          new Annotation[0], MediaType.TEXT_XML_TYPE, 
                          new MetadataMap<String, Object>(), bos);
-        String expected = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
-            + "<ns1:SuperBooks xmlns:ns1=\"http://superbooks\">"
+        String expected = "<ns1:SuperBooks xmlns:ns1=\"http://superbooks\">"
             + "<ns1:SuperBook xmlns:ns2=\"http://books\" xmlns:ns1=\"http://superbooks\"><id>123</id>"
             + "<name>CXF in Action</name><superId>124</superId></ns1:SuperBook></ns1:SuperBooks>";
         assertEquals(expected, bos.toString());
@@ -471,7 +441,7 @@ public class JAXBElementProviderTest extends Assert {
                          new MetadataMap<String, Object>(), bos);
         assertTrue(bos.toString().contains("thebook2"));
         assertTrue(bos.toString().contains("http://superbooks"));
-        System.out.println(bos.toString());
+        
         ByteArrayInputStream is = new ByteArrayInputStream(bos.toByteArray());
         Book2 book2 = 
             (Book2)provider.readFrom(
@@ -812,8 +782,7 @@ public class JAXBElementProviderTest extends Assert {
         provider.writeTo(list, ArrayList.class, type,
             new Annotation[0], MediaType.TEXT_XML_TYPE, new MetadataMap<String, Object>(), bos);
     
-        String expected = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
-            + "<tagholders><tagholder><attr>attribute</attr>"
+        String expected = "<tagholders><tagholder><attr>attribute</attr>"
             + "<thetag><group>B</group><name>A</name></thetag></tagholder></tagholders>";
         assertEquals(expected, bos.toString());
     }
@@ -1024,20 +993,6 @@ public class JAXBElementProviderTest extends Assert {
         doReadUnqualifiedCollection(data, "setBooks", List.class);
     }
     
-    @Test
-    @SuppressWarnings("unchecked")
-    public void testReadMalformedXML() throws Exception {
-        JAXBElementProvider provider = new JAXBElementProvider();
-        try {
-            provider.readFrom((Class)Book.class, Book.class,
-                       new Annotation[0], MediaType.TEXT_XML_TYPE, new MetadataMap<String, String>(), 
-                       new ByteArrayInputStream("<Book>".getBytes()));
-            fail("404 is expected");
-        } catch (WebApplicationException ex) {
-            assertEquals(400, ex.getResponse().getStatus());
-        }
-    }
-    
     @SuppressWarnings("unchecked")
     private void doReadUnqualifiedCollection(String data, String mName, Class<?> type) throws Exception {
         JAXBElementProvider provider = new JAXBElementProvider();
