@@ -20,6 +20,10 @@
 package org.apache.cxf.ws.addressing;
 
 
+import java.io.IOException;
+import java.io.InputStream;
+import java.util.HashSet;
+import java.util.Set;
 import java.util.UUID;
 import java.util.concurrent.Executor;
 import java.util.logging.Level;
@@ -43,16 +47,22 @@ import org.apache.cxf.interceptor.Fault;
 import org.apache.cxf.interceptor.InterceptorChain;
 import org.apache.cxf.interceptor.OutgoingChainInterceptor;
 import org.apache.cxf.io.DelegatingInputStream;
+import org.apache.cxf.jaxb.JAXBContextCache;
+import org.apache.cxf.jaxb.JAXBContextCache.CachedContextAndSchemas;
 import org.apache.cxf.message.Exchange;
 import org.apache.cxf.message.Message;
 import org.apache.cxf.message.MessageImpl;
 import org.apache.cxf.service.model.BindingFaultInfo;
 import org.apache.cxf.service.model.BindingOperationInfo;
+import org.apache.cxf.service.model.EndpointInfo;
 import org.apache.cxf.service.model.Extensible;
 import org.apache.cxf.service.model.FaultInfo;
 import org.apache.cxf.service.model.MessageInfo;
 import org.apache.cxf.transport.Conduit;
+import org.apache.cxf.transport.ConduitInitiator;
+import org.apache.cxf.transport.ConduitInitiatorManager;
 import org.apache.cxf.transport.Destination;
+import org.apache.cxf.transport.MessageObserver;
 import org.apache.cxf.workqueue.OneShotAsyncExecutor;
 import org.apache.cxf.workqueue.SynchronousExecutor;
 import org.apache.cxf.workqueue.WorkQueueManager;
@@ -87,6 +97,7 @@ public final class ContextUtils {
     private static final String URN_UUID = "urn:uuid:";
     
     private static JAXBContext jaxbContext;
+    private static Set<Class<?>> jaxbContextClasses;
      
     /**
      * Used by MAPAggregator to cache bad MAP fault name
@@ -316,6 +327,18 @@ public final class ContextUtils {
                || Names.WSA_ANONYMOUS_ADDRESS.equals(ref.getAddress().getValue())
                || Names.WSA_NONE_ADDRESS.equals(ref.getAddress().getValue());
     }
+    
+    /**
+     * Helper method to determine if an EPR address is none.
+     *
+     * @param ref the EPR under test
+     * @return true if the address is generic
+     */
+    public static boolean isNoneAddress(EndpointReferenceType ref) {
+        return ref != null 
+               && ref.getAddress() != null
+               && Names.WSA_NONE_ADDRESS.equals(ref.getAddress().getValue());
+    }
 
     /**
      * Helper method to determine if an MAPs Action is empty (a null action
@@ -364,6 +387,9 @@ public final class ContextUtils {
             }
             
             try {
+                if (reference == null) {
+                    reference = ContextUtils.getNoneEndpointReference();
+                }
                 exchange.setOutMessage(partialResponse);
                 Conduit backChannel = target.getBackChannel(inMessage,
                                                             partialResponse,
@@ -392,18 +418,23 @@ public final class ContextUtils {
                         chain.reset();                        
                     }
                     exchange.put(ConduitSelector.class, new NullConduitSelector());
-                    if (fullResponse != null) {
-                        exchange.setOutMessage(fullResponse);
-                    } else {
+                    
+                    if (fullResponse == null) {
                         fullResponse = createMessage(exchange);
-                        exchange.setOutMessage(fullResponse);
                     }
+                    exchange.setOutMessage(fullResponse);
+                    
+                    Destination destination = createDecoupledDestination(
+                        exchange, 
+                        reference);
+                    exchange.setDestination(destination);
+                         
                     
                     if (retrieveAsyncPostResponseDispatch(inMessage)) {
                         //need to suck in all the data from the input stream as
                         //the transport might discard any data on the stream when this 
                         //thread unwinds or when the empty response is sent back
-                        DelegatingInputStream in = inMessage.get(DelegatingInputStream.class);
+                        DelegatingInputStream in = inMessage.getContent(DelegatingInputStream.class);
                         if (in != null) {
                             in.cacheInput();
                         }
@@ -412,6 +443,7 @@ public final class ContextUtils {
                         // has been sent (i.e. to a oneway, or a partial response
                         // to a decoupled twoway)
                         
+        
                         // pause dispatch on current thread ...
                         inMessage.getInterceptorChain().pause();
 
@@ -429,6 +461,51 @@ public final class ContextUtils {
         }
     }
 
+    public static Destination createDecoupledDestination(
+        Exchange exchange, final EndpointReferenceType reference) {
+
+        final EndpointInfo ei = exchange.get(Endpoint.class).getEndpointInfo();
+        return new Destination() {
+            public EndpointReferenceType getAddress() {
+                return reference;
+            }
+            public Conduit getBackChannel(Message inMessage, Message partialResponse,
+                                          EndpointReferenceType address) throws IOException {
+                Bus bus = inMessage.getExchange().get(Bus.class);
+                //this is a response targeting a decoupled endpoint.   Treat it as a oneway so
+                //we don't wait for a response.
+                inMessage.getExchange().setOneWay(true);
+                ConduitInitiator conduitInitiator 
+                    = bus.getExtension(ConduitInitiatorManager.class)
+                        .getConduitInitiatorForUri(reference.getAddress().getValue());
+                if (conduitInitiator != null) {
+                    Conduit c = conduitInitiator.getConduit(ei, reference);
+                    // ensure decoupled back channel input stream is closed
+                    c.setMessageObserver(new MessageObserver() {
+                        public void onMessage(Message m) {
+                            InputStream is = m.getContent(InputStream.class);
+                            if (is != null) {
+                                try {
+                                    is.close();
+                                } catch (Exception e) {
+                                    // ignore
+                                }
+                            }
+                        }
+                    });
+                    return c;
+                }
+                return null;
+            }
+            public MessageObserver getMessageObserver() {
+                return null;
+            }
+            public void shutdown() {
+            }
+            public void setMessageObserver(MessageObserver observer) {
+            }
+        };
+    }
     
     /**
      * Propogate inbound MAPs onto full reponse & fault messages.
@@ -650,11 +727,15 @@ public final class ContextUtils {
      */
     public static JAXBContext getJAXBContext() throws JAXBException {
         synchronized (ContextUtils.class) {
-            if (jaxbContext == null) {
-                jaxbContext =
-                    JAXBContext.newInstance(
-                        WSA_OBJECT_FACTORY.getClass().getPackage().getName(),
-                        WSA_OBJECT_FACTORY.getClass().getClassLoader());
+            if (jaxbContext == null || jaxbContextClasses == null) {
+                Set<Class<?>> tmp = new HashSet<Class<?>>();
+                JAXBContextCache.addPackage(tmp, WSA_OBJECT_FACTORY.getClass().getPackage().getName(), 
+                                            WSA_OBJECT_FACTORY.getClass().getClassLoader());
+                JAXBContextCache.scanPackages(tmp);
+                CachedContextAndSchemas ccs 
+                    = JAXBContextCache.getCachedContextAndSchemas(tmp, null, null, null, false);
+                jaxbContextClasses = ccs.getClasses();
+                jaxbContext = ccs.getContext();
             }
         }
         return jaxbContext;
@@ -668,6 +749,7 @@ public final class ContextUtils {
     public static void setJAXBContext(JAXBContext ctx) throws JAXBException {
         synchronized (ContextUtils.class) {
             jaxbContext = ctx;
+            jaxbContextClasses = new HashSet<Class<?>>();
         }
     }
     
@@ -806,7 +888,7 @@ public final class ContextUtils {
         if (msgInfo != null
             && msgInfo.getExtensionAttributes() != null) {
             String attr = getAction(msgInfo);
-            if (attr != null) {
+            if (!StringUtils.isEmpty(attr)) {
                 action = attr;
                 msgInfo.setProperty(ACTION, action);
             }
@@ -820,6 +902,9 @@ public final class ContextUtils {
             o = ext.getExtensionAttributes().get(new QName(Names.WSA_NAMESPACE_WSDL_METADATA,
                                                            Names.WSAW_ACTION_NAME));
         }
+        if (o == null) {
+            o = ext.getExtensionAttributes().get(new QName(JAXWSAConstants.NS_WSA, Names.WSAW_ACTION_NAME));
+        }
         if (o == null) {
             o = ext.getExtensionAttributes().get(new QName(Names.WSA_NAMESPACE_WSDL_NAME_OLD,
                                                    Names.WSAW_ACTION_NAME));
@@ -846,7 +931,7 @@ public final class ContextUtils {
      * @param message the current message
      * @return the Method from the BindingOperationInfo
      */
-    private static Message createMessage(Exchange exchange) {
+    public static Message createMessage(Exchange exchange) {
         Endpoint ep = exchange.get(Endpoint.class);
         Message msg = null;
         if (ep != null) {
