@@ -112,10 +112,8 @@ import org.apache.ws.security.components.crypto.CryptoType;
 import org.apache.ws.security.conversation.ConversationException;
 import org.apache.ws.security.conversation.dkalgo.P_SHA1;
 import org.apache.ws.security.handler.RequestData;
-import org.apache.ws.security.message.token.BinarySecurity;
 import org.apache.ws.security.message.token.Reference;
 import org.apache.ws.security.processor.EncryptedKeyProcessor;
-import org.apache.ws.security.processor.X509Util;
 import org.apache.ws.security.util.Base64;
 import org.apache.ws.security.util.WSSecurityUtil;
 import org.apache.ws.security.util.XmlSchemaDateFormat;
@@ -129,51 +127,49 @@ import org.apache.xml.security.keys.content.keyvalues.RSAKeyValue;
 public class STSClient implements Configurable, InterceptorProvider {
     private static final Logger LOG = LogUtils.getL7dLogger(STSClient.class);
     
-    protected Bus bus;
-    protected String name = "default.sts-client";
-    protected Client client;
-    protected String location;
+    Bus bus;
+    String name = "default.sts-client";
+    Client client;
+    String location;
 
-    protected String wsdlLocation;
-    protected QName serviceName;
-    protected QName endpointName;
+    String wsdlLocation;
+    QName serviceName;
+    QName endpointName;
 
-    protected Policy policy;
-    protected String soapVersion = SoapBindingConstants.SOAP11_BINDING_ID;
-    protected int keySize = 256;
-    protected boolean requiresEntropy = true;
-    protected Element template;
-    protected Element claims;
-    protected AlgorithmSuite algorithmSuite;
-    protected String namespace = STSUtils.WST_NS_05_12;
-    protected String addressingNamespace;
-    protected Object onBehalfOf;
-    protected boolean enableAppliesTo = true;
+    Policy policy;
+    String soapVersion = SoapBindingConstants.SOAP11_BINDING_ID;
+    int keySize = 256;
+    boolean requiresEntropy = true;
+    Element template;
+    AlgorithmSuite algorithmSuite;
+    String namespace = STSUtils.WST_NS_05_12;
+    String addressingNamespace;
+    Object onBehalfOf;
+    boolean enableAppliesTo = true;
 
-    protected boolean useCertificateForConfirmationKeyInfo;
-    protected boolean isSecureConv;
-    protected boolean isSpnego;
-    protected boolean enableLifetime;
-    protected int ttl = 300;
+    boolean useCertificateForConfirmationKeyInfo;
+    boolean isSecureConv;
+    boolean enableLifetime;
+    int ttl = 300;
     
-    protected Object actAs;
-    protected String tokenType;
-    protected String keyType;
-    protected boolean sendKeyType = true;
-    protected Message message;
-    protected String context;
+    Object actAs;
+    String tokenType;
+    String keyType;
+    boolean sendKeyType = true;
+    Message message;
+    String context;
 
-    protected Map<String, Object> ctx = new HashMap<String, Object>();
+    Map<String, Object> ctx = new HashMap<String, Object>();
     
-    protected List<Interceptor<? extends Message>> in 
+    List<Interceptor<? extends Message>> in 
         = new ModCountCopyOnWriteArrayList<Interceptor<? extends Message>>();
-    protected List<Interceptor<? extends Message>> out 
+    List<Interceptor<? extends Message>> out 
         = new ModCountCopyOnWriteArrayList<Interceptor<? extends Message>>();
-    protected List<Interceptor<? extends Message>> outFault  
+    List<Interceptor<? extends Message>> outFault  
         = new ModCountCopyOnWriteArrayList<Interceptor<? extends Message>>();
-    protected List<Interceptor<? extends Message>> inFault 
+    List<Interceptor<? extends Message>> inFault 
         = new ModCountCopyOnWriteArrayList<Interceptor<? extends Message>>();
-    protected List<AbstractFeature> features;
+    List<AbstractFeature> features;
 
     public STSClient(Bus b) {
         bus = b;
@@ -274,14 +270,6 @@ public class STSClient implements Configurable, InterceptorProvider {
         this.isSecureConv = secureConv;
     }
     
-    public boolean isSpnego() {
-        return isSpnego;
-    }
-
-    public void setSpnego(boolean spnego) {
-        this.isSpnego = spnego;
-    }
-    
     public boolean isEnableAppliesTo() {
         return enableAppliesTo;
     }
@@ -359,10 +347,6 @@ public class STSClient implements Configurable, InterceptorProvider {
         this.tokenType = tokenType;
     }
     
-    public String getTokenType() {
-        return tokenType;
-    }
-    
     public void setSendKeyType(boolean sendKeyType) {
         this.sendKeyType = sendKeyType;
     }
@@ -421,7 +405,8 @@ public class STSClient implements Configurable, InterceptorProvider {
         }
         return client;
     }
-    protected void createClient() throws BusException, EndpointException {
+
+    private void createClient() throws BusException, EndpointException {
         if (client != null) {
             return;
         }
@@ -458,7 +443,7 @@ public class STSClient implements Configurable, InterceptorProvider {
         }
     }
 
-    protected BindingOperationInfo findOperation(String suffix) {
+    private BindingOperationInfo findOperation(String suffix) {
         BindingInfo bi = client.getEndpoint().getBinding().getBindingInfo();
         for (BindingOperationInfo boi : bi.getOperations()) {
             SoapOperationInfo soi = boi.getExtensor(SoapOperationInfo.class);
@@ -489,24 +474,15 @@ public class STSClient implements Configurable, InterceptorProvider {
     }
 
     public SecurityToken requestSecurityToken(String appliesTo) throws Exception {
-        return requestSecurityToken(appliesTo, null);
-    }
-    
-    public SecurityToken requestSecurityToken(String appliesTo, String binaryExchange) throws Exception {
         String action = null;
         if (isSecureConv) {
             action = namespace + "/RST/SCT";
         }
-        return requestSecurityToken(appliesTo, action, "/Issue", null, binaryExchange);
-    }
-    
-    public SecurityToken requestSecurityToken(String appliesTo, String action, String requestType,
-            SecurityToken target) throws Exception {
-        return requestSecurityToken(appliesTo, action, requestType, target, null);
+        return requestSecurityToken(appliesTo, action, "/Issue", null);
     }
 
     public SecurityToken requestSecurityToken(String appliesTo, String action, String requestType,
-                                              SecurityToken target, String binaryExchange) throws Exception {
+                                              SecurityToken target) throws Exception {
         createClient();
         BindingOperationInfo boi = findOperation("/RST/Issue");
 
@@ -552,25 +528,12 @@ public class STSClient implements Configurable, InterceptorProvider {
                 writer.writeEndElement();
             }
         }
-        
-        if (isSpnego) {
-            tokenType = STSUtils.getTokenTypeSCT(namespace);
-            sendKeyType = false;
-        }
 
         addRequestType(requestType, writer);
         if (enableAppliesTo) {
             addAppliesTo(writer, appliesTo);
         }
-        
-        addClaims(writer);
-        
-        Element onBehalfOfToken = getOnBehalfOfToken();
-        if (onBehalfOfToken != null) {
-            writer.writeStartElement("wst", "OnBehalfOf", namespace);
-            StaxUtils.copy(onBehalfOfToken, writer);
-            writer.writeEndElement();
-        }
+        addOnBehalfOf(writer);
         if (sptt == null) {
             addTokenType(writer);
         }
@@ -594,8 +557,6 @@ public class STSClient implements Configurable, InterceptorProvider {
             crypto = createCrypto(false);
             cert = getCert(crypto);
             writeElementsForRSTPublicKey(writer, cert);
-        } else if (isSpnego) {
-            addKeySize(keySize, writer);
         }
         
         if (target != null) {
@@ -608,16 +569,7 @@ public class STSClient implements Configurable, InterceptorProvider {
             writer.writeEndElement();
         }
         
-        if (binaryExchange != null) {
-            addBinaryExchange(binaryExchange, writer);
-        }
-
-        Element actAsSecurityToken = getActAsToken();
-        if (actAsSecurityToken != null) {
-            writer.writeStartElement(STSUtils.WST_NS_08_02, "ActAs");
-            StaxUtils.copy(actAsSecurityToken, writer);
-            writer.writeEndElement();
-        }
+        addActAs(writer);
         
         writer.writeEndElement();
 
@@ -637,48 +589,14 @@ public class STSClient implements Configurable, InterceptorProvider {
         return token;
     }
     
-    /**
-     * Get the "OnBehalfOf" element to be sent to the STS.
-     */
-    public Element getOnBehalfOfToken() throws Exception {
-        return getDelegationSecurityToken(this.onBehalfOf);
-    }
-    
-    /**
-     * Get the "ActAs" element to be sent to the STS.
-     */
-    public Element getActAsToken() throws Exception {
-        return getDelegationSecurityToken(this.actAs);
-    }
-    
-    protected Element getDelegationSecurityToken(Object delegationObject) throws Exception {
-        if (delegationObject != null) {
-            final boolean isString = delegationObject instanceof String;
-            final boolean isElement = delegationObject instanceof Element; 
-            final boolean isCallbackHandler = delegationObject instanceof CallbackHandler;
-            if (isString || isElement || isCallbackHandler) {
-                if (isString) {
-                    final Document doc =
-                        DOMUtils.readXml(new StringReader((String) delegationObject));
-                    return doc.getDocumentElement();
-                } else if (isElement) {
-                    return (Element) delegationObject;
-                } else {
-                    DelegationCallback callback = new DelegationCallback(message);
-                    ((CallbackHandler)delegationObject).handle(new Callback[]{callback});
-                    return callback.getToken();
-                }
-            }
-        }
-        return null;
-    }
-    
-    protected byte[] writeElementsForRSTSymmetricKey(W3CDOMStreamWriter writer,
+    private byte[] writeElementsForRSTSymmetricKey(W3CDOMStreamWriter writer,
             boolean wroteKeySize) throws Exception {
         byte[] requestorEntropy = null;
 
         if (!wroteKeySize && (!isSecureConv || keySize != 256)) {
-            addKeySize(keySize, writer);
+            writer.writeStartElement("wst", "KeySize", namespace);
+            writer.writeCharacters(Integer.toString(keySize));
+            writer.writeEndElement();
         }
 
         if (requiresEntropy) {
@@ -703,7 +621,7 @@ public class STSClient implements Configurable, InterceptorProvider {
     }
 
 
-    protected void writeElementsForRSTPublicKey(W3CDOMStreamWriter writer,
+    private void writeElementsForRSTPublicKey(W3CDOMStreamWriter writer,
             X509Certificate cert) throws Exception {
         writer.writeStartElement("wst", "UseKey", namespace);
         writer.writeStartElement("dsig", "KeyInfo", "http://www.w3.org/2000/09/xmldsig#");
@@ -735,31 +653,43 @@ public class STSClient implements Configurable, InterceptorProvider {
         writer.writeEndElement();
         writer.writeEndElement();
     }
-    
-    protected void addBinaryExchange(
-        String binaryExchange, 
-        W3CDOMStreamWriter writer
-    ) throws XMLStreamException {
-        writer.writeStartElement("wst", "BinaryExchange", namespace);
-        writer.writeAttribute("EncodingType", BinarySecurity.BASE64_ENCODING);
-        writer.writeAttribute("ValueType", namespace + "/spnego");
-        writer.writeCharacters(binaryExchange);
-        writer.writeEndElement();
-    }
-    
-    protected void addKeySize(int keysize, W3CDOMStreamWriter writer) throws XMLStreamException {
-        writer.writeStartElement("wst", "KeySize", namespace);
-        writer.writeCharacters(Integer.toString(keysize));
-        writer.writeEndElement();
-    }
 
-    protected void addRequestType(String requestType, W3CDOMStreamWriter writer) throws XMLStreamException {
+    private void addRequestType(String requestType, W3CDOMStreamWriter writer) throws XMLStreamException {
         writer.writeStartElement("wst", "RequestType", namespace);
         writer.writeCharacters(namespace + requestType);
         writer.writeEndElement();
     }
     
-    protected Element getDocumentElement(DOMSource ds) {
+    private void addOnBehalfOf(W3CDOMStreamWriter writer) throws Exception {
+        if (this.onBehalfOf != null) {
+            final boolean isString = this.onBehalfOf instanceof String;
+            final boolean isElement = this.onBehalfOf instanceof Element; 
+            final boolean isCallbackHandler = this.onBehalfOf instanceof CallbackHandler;
+            if (isString || isElement || isCallbackHandler) {
+                final Element tokenElement;
+                
+                if (isString) {
+                    final Document acAsDoc =
+                        DOMUtils.readXml(new StringReader((String) this.onBehalfOf));
+                    tokenElement = acAsDoc.getDocumentElement();
+                } else if (isElement) {
+                    tokenElement = (Element)this.onBehalfOf;
+                } else {
+                    DelegationCallback callback = new DelegationCallback(message);
+                    ((CallbackHandler)onBehalfOf).handle(new Callback[]{callback});
+                    tokenElement = callback.getToken();
+                }
+                
+                if (tokenElement != null) {
+                    writer.writeStartElement("wst", "OnBehalfOf", namespace);
+                    StaxUtils.copy(tokenElement, writer);
+                    writer.writeEndElement();
+                }
+            }
+        }
+    }
+    
+    private Element getDocumentElement(DOMSource ds) {
         Node nd = ds.getNode();
         if (nd instanceof Document) {
             nd = ((Document)nd).getDocumentElement();
@@ -775,7 +705,7 @@ public class STSClient implements Configurable, InterceptorProvider {
         requestSecurityToken(tok.getIssuerAddress(), action, "/Renew", tok);
     }
 
-    protected PrimitiveAssertion getAddressingAssertion() {
+    private PrimitiveAssertion getAddressingAssertion() {
         String ns = "http://schemas.xmlsoap.org/ws/2004/08/addressing/policy";
         String local = "UsingAddressing";
         if ("http://www.w3.org/2005/08/addressing".equals(addressingNamespace)) {
@@ -793,7 +723,7 @@ public class STSClient implements Configurable, InterceptorProvider {
         return validateSecurityToken(tok, validateTokenType);
     }
     
-    protected List<SecurityToken> validateSecurityToken(SecurityToken tok, String tokentype) 
+    private List<SecurityToken> validateSecurityToken(SecurityToken tok, String tokentype) 
         throws Exception {
         createClient();
         
@@ -984,11 +914,11 @@ public class STSClient implements Configurable, InterceptorProvider {
         }
     }
     
-    protected boolean useSecondaryParameters() {
+    private boolean useSecondaryParameters() {
         return !STSUtils.WST_NS_05_02.equals(namespace);
     }
 
-    protected String writeKeyType(W3CDOMStreamWriter writer, String keyTypeToWrite) 
+    private String writeKeyType(W3CDOMStreamWriter writer, String keyTypeToWrite) 
         throws XMLStreamException {
         if (isSecureConv) {
             if (keyTypeToWrite == null) {
@@ -1010,7 +940,7 @@ public class STSClient implements Configurable, InterceptorProvider {
         return keyTypeToWrite;
     }
 
-    protected X509Certificate getCert(Crypto crypto) throws Exception {
+    private X509Certificate getCert(Crypto crypto) throws Exception {
         String alias = (String)getProperty(SecurityConstants.STS_TOKEN_USERNAME);
         if (alias == null) {
             alias = crypto.getDefaultX509Identifier();
@@ -1028,7 +958,7 @@ public class STSClient implements Configurable, InterceptorProvider {
         return certs[0];
     }
 
-    protected void addLifetime(XMLStreamWriter writer) throws XMLStreamException {
+    private void addLifetime(XMLStreamWriter writer) throws XMLStreamException {
         Date creationTime = new Date();
         Date expirationTime = new Date();
         expirationTime.setTime(creationTime.getTime() + (ttl * 1000L));
@@ -1046,7 +976,7 @@ public class STSClient implements Configurable, InterceptorProvider {
         writer.writeEndElement();
     }
 
-    protected void addAppliesTo(XMLStreamWriter writer, String appliesTo) throws XMLStreamException {
+    private void addAppliesTo(XMLStreamWriter writer, String appliesTo) throws XMLStreamException {
         if (appliesTo != null && addressingNamespace != null) {
             writer.writeStartElement("wsp", "AppliesTo", "http://schemas.xmlsoap.org/ws/2004/09/policy");
             writer.writeNamespace("wsp", "http://schemas.xmlsoap.org/ws/2004/09/policy");
@@ -1060,21 +990,44 @@ public class STSClient implements Configurable, InterceptorProvider {
         }
     }
 
-    protected void addTokenType(XMLStreamWriter writer) throws XMLStreamException {
+    private void addTokenType(XMLStreamWriter writer) throws XMLStreamException {
         if (tokenType != null) {
             writer.writeStartElement("wst", "TokenType", namespace);
             writer.writeCharacters(tokenType);
             writer.writeEndElement();
         }
     }
-    
-    protected void addClaims(XMLStreamWriter writer) throws XMLStreamException {
-        if (claims != null) {
-            StaxUtils.copy(claims, writer);
+
+    private void addActAs(W3CDOMStreamWriter writer) throws Exception {
+        if (this.actAs != null) {
+            final boolean isString = this.actAs instanceof String;
+            final boolean isElement = this.actAs instanceof Element; 
+            final boolean isCallbackHandler = this.actAs instanceof CallbackHandler;
+            if (isString || isElement || isCallbackHandler) {
+                final Element tokenElement;
+                
+                if (isString) {
+                    final Document acAsDoc =
+                        DOMUtils.readXml(new StringReader((String) this.actAs));
+                    tokenElement = acAsDoc.getDocumentElement();
+                } else if (isElement) {
+                    tokenElement = (Element) this.actAs;
+                } else {
+                    DelegationCallback callback = new DelegationCallback(message);
+                    ((CallbackHandler)actAs).handle(new Callback[]{callback});
+                    tokenElement = callback.getToken();
+                }
+                
+                if (tokenElement != null) {
+                    writer.writeStartElement(STSUtils.WST_NS_08_02, "ActAs");
+                    StaxUtils.copy(tokenElement, writer);
+                    writer.writeEndElement();
+                }
+            }
         }
     }
 
-    protected SecurityToken createSecurityToken(Element el, byte[] requestorEntropy)
+    private SecurityToken createSecurityToken(Element el, byte[] requestorEntropy)
         throws WSSecurityException {
 
         if ("RequestSecurityTokenResponseCollection".equals(el.getLocalName())) {
@@ -1177,47 +1130,26 @@ public class STSClient implements Configurable, InterceptorProvider {
         return token;
     }
     
-    protected byte[] decryptKey(Element child) throws TrustException, WSSecurityException {
-        String encryptionAlgorithm = X509Util.getEncAlgo(child);
-        // For the SPNEGO case just return the decoded cipher value and decrypt it later
-        if (encryptionAlgorithm != null && encryptionAlgorithm.endsWith("spnego#GSS_Wrap")) {
-            // Get the CipherValue
-            Element tmpE = 
-                WSSecurityUtil.getDirectChildElement(child, "CipherData", WSConstants.ENC_NS);
-            byte[] cipherValue = null;
-            if (tmpE != null) {
-                tmpE = 
-                    WSSecurityUtil.getDirectChildElement(tmpE, "CipherValue", WSConstants.ENC_NS);
-                if (tmpE != null) {
-                    String content = DOMUtils.getContent(tmpE);
-                    cipherValue = Base64.decode(content);
-                }
-            }
-            if (cipherValue == null) {
-                throw new WSSecurityException(WSSecurityException.INVALID_SECURITY, "noCipher");
-            }
-            return cipherValue;
-        } else {
-            try {
-                EncryptedKeyProcessor proc = new EncryptedKeyProcessor();
-                WSDocInfo docInfo = new WSDocInfo(child.getOwnerDocument());
-                RequestData data = new RequestData();
-                data.setWssConfig(WSSConfig.getNewInstance());
-                data.setDecCrypto(createCrypto(true));
-                data.setCallbackHandler(createHandler());
-                List<WSSecurityEngineResult> result =
-                    proc.handleToken(child, data, docInfo);
-                return 
-                    (byte[])result.get(0).get(
-                        WSSecurityEngineResult.TAG_SECRET
-                    );
-            } catch (IOException e) {
-                throw new TrustException("ENCRYPTED_KEY_ERROR", LOG, e);
-            }
+    private byte[] decryptKey(Element child) throws TrustException, WSSecurityException {
+        try {
+            EncryptedKeyProcessor proc = new EncryptedKeyProcessor();
+            WSDocInfo docInfo = new WSDocInfo(child.getOwnerDocument());
+            RequestData data = new RequestData();
+            data.setWssConfig(WSSConfig.getNewInstance());
+            data.setDecCrypto(createCrypto(true));
+            data.setCallbackHandler(createHandler());
+            List<WSSecurityEngineResult> result =
+                proc.handleToken(child, data, docInfo);
+            return 
+                (byte[])result.get(0).get(
+                    WSSecurityEngineResult.TAG_SECRET
+                );
+        } catch (IOException e) {
+            throw new TrustException("ENCRYPTED_KEY_ERROR", LOG, e);
         }
     }
 
-    protected CallbackHandler createHandler() {
+    private CallbackHandler createHandler() {
         Object o = getProperty(SecurityConstants.CALLBACK_HANDLER);
         if (o instanceof String) {
             try {
@@ -1230,7 +1162,7 @@ public class STSClient implements Configurable, InterceptorProvider {
         return (CallbackHandler)o;
     }
 
-    protected Object getProperty(String s) {
+    private Object getProperty(String s) {
         Object o = ctx.get(s);
         if (o == null) {
             o = client.getEndpoint().getEndpointInfo().getProperty(s);
@@ -1244,7 +1176,7 @@ public class STSClient implements Configurable, InterceptorProvider {
         return o;
     }
 
-    protected Crypto createCrypto(boolean decrypt) throws IOException, WSSecurityException {
+    private Crypto createCrypto(boolean decrypt) throws IOException, WSSecurityException {
         Crypto crypto = (Crypto)getProperty(SecurityConstants.STS_TOKEN_CRYPTO + (decrypt ? ".decrypt" : ""));
         if (crypto != null) {
             return crypto;
@@ -1284,7 +1216,7 @@ public class STSClient implements Configurable, InterceptorProvider {
         return null;
     }
 
-    protected String findID(Element rar, Element rur, Element rst) {
+    private String findID(Element rar, Element rur, Element rst) {
         String id = null;
         if (rst != null) {
             QName elName = DOMUtils.getElementQName(rst);
@@ -1305,13 +1237,13 @@ public class STSClient implements Configurable, InterceptorProvider {
         if (id == null && rur != null) {
             id = this.getIDFromSTR(rur);
         }
-        if (id == null && rst != null) {
+        if (id == null) {
             id = rst.getAttributeNS(WSConstants.WSU_NS, "Id");
         }
         return id;
     }
 
-    protected String getIDFromSTR(Element el) {
+    private String getIDFromSTR(Element el) {
         Element child = DOMUtils.getFirstElement(el);
         if (child == null) {
             return null;
@@ -1332,10 +1264,6 @@ public class STSClient implements Configurable, InterceptorProvider {
     public void setTemplate(Element rstTemplate) {
         template = rstTemplate;
     }
-
-    public void setClaims(Element rstClaims) {
-        claims = rstClaims;
-    }
     
     public List<Interceptor<? extends Message>> getOutFaultInterceptors() {
         if (client != null) {
