@@ -44,6 +44,7 @@ import org.w3c.dom.Document;
 import org.apache.cxf.binding.soap.SoapMessage;
 import org.apache.cxf.binding.soap.interceptor.MustUnderstandInterceptor;
 import org.apache.cxf.binding.soap.saaj.SAAJInInterceptor;
+import org.apache.cxf.helpers.CastUtils;
 import org.apache.cxf.helpers.DOMUtils.NullResolver;
 import org.apache.cxf.helpers.XMLUtils;
 import org.apache.cxf.interceptor.Interceptor;
@@ -61,6 +62,7 @@ import org.apache.ws.security.WSSecurityEngineResult;
 import org.apache.ws.security.WSUsernameTokenPrincipal;
 import org.apache.ws.security.handler.WSHandlerConstants;
 import org.apache.ws.security.handler.WSHandlerResult;
+import org.apache.ws.security.util.WSSecurityUtil;
 
 import org.junit.Test;
 
@@ -89,210 +91,91 @@ public class WSS4JInOutTest extends AbstractSecurityTest {
         assertTrue(output.contains("MustUnderstandInterceptor, SAAJInInterceptor, WSS4JInInterceptor"));
     }
     
+    
     @Test
     public void testSignature() throws Exception {
-        Document doc = readDocument("wsse-request-clean.xml");
-
-        WSS4JOutInterceptor ohandler = new WSS4JOutInterceptor();
-        PhaseInterceptor<SoapMessage> handler = ohandler.createEndingInterceptor();
-
-        SoapMessage msg = new SoapMessage(new MessageImpl());
-        Exchange ex = new ExchangeImpl();
-        ex.setInMessage(msg);
+        Map<String, String> outProperties = new HashMap<String, String>();
+        outProperties.put(WSHandlerConstants.ACTION, WSHandlerConstants.SIGNATURE);
+        outProperties.put(WSHandlerConstants.SIG_PROP_FILE, "outsecurity.properties");
+        outProperties.put(WSHandlerConstants.USER, "myalias");
+        outProperties.put("password", "myAliasPassword");
         
-        SOAPMessage saajMsg = MessageFactory.newInstance().createMessage();
-        SOAPPart part = saajMsg.getSOAPPart();
-        part.setContent(new DOMSource(doc));
-        saajMsg.saveChanges();
-
-        msg.setContent(SOAPMessage.class, saajMsg);
-
-        msg.put(WSHandlerConstants.ACTION, WSHandlerConstants.SIGNATURE);
-        msg.put(WSHandlerConstants.SIG_PROP_FILE, "META-INF/cxf/outsecurity.properties");
-        msg.put(WSHandlerConstants.USER, "myalias");
-        msg.put("password", "myAliasPassword");
-
-        handler.handleMessage(msg);
-
-        doc = part;
+        Map<String, String> inProperties = new HashMap<String, String>();
+        inProperties.put(WSHandlerConstants.ACTION, WSHandlerConstants.SIGNATURE);
+        inProperties.put(WSHandlerConstants.SIG_PROP_FILE, "insecurity.properties");
         
-        assertValid("//wsse:Security", doc);
-        assertValid("//wsse:Security/ds:Signature", doc);
+        List<String> xpaths = new ArrayList<String>();
+        xpaths.add("//wsse:Security");
+        xpaths.add("//wsse:Security/ds:Signature");
 
-        byte[] docbytes = getMessageBytes(doc);
-        XMLStreamReader reader = StaxUtils.createXMLStreamReader(new ByteArrayInputStream(docbytes));
-
-        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
-
-        dbf.setValidating(false);
-        dbf.setIgnoringComments(false);
-        dbf.setIgnoringElementContentWhitespace(true);
-        dbf.setNamespaceAware(true);
-
-        DocumentBuilder db = dbf.newDocumentBuilder();
-        db.setEntityResolver(new NullResolver());
-        doc = StaxUtils.read(db, reader, false);
-
-        WSS4JInInterceptor inHandler = new WSS4JInInterceptor();
-
-        SoapMessage inmsg = new SoapMessage(new MessageImpl());
-        ex.setInMessage(inmsg);
-        inmsg.setContent(SOAPMessage.class, saajMsg);
-
-        inHandler.setProperty(WSHandlerConstants.ACTION, WSHandlerConstants.SIGNATURE);
-        inHandler.setProperty(WSHandlerConstants.SIG_PROP_FILE, "META-INF/cxf/insecurity.properties");
-
-        inHandler.handleMessage(inmsg);
-        
-        WSSecurityEngineResult result = 
-            (WSSecurityEngineResult) inmsg.get(WSS4JInInterceptor.SIGNATURE_RESULT);
-        assertNotNull(result);
-        X509Certificate certificate = (X509Certificate)result
-            .get(WSSecurityEngineResult.TAG_X509_CERTIFICATE);
+        List<WSHandlerResult> handlerResults = 
+            getResults(makeInvocation(outProperties, xpaths, inProperties));
+        WSSecurityEngineResult actionResult =
+            WSSecurityUtil.fetchActionResult(handlerResults.get(0).getResults(), WSConstants.SIGN);
+         
+        X509Certificate certificate = 
+            (X509Certificate) actionResult.get(WSSecurityEngineResult.TAG_X509_CERTIFICATE);
         assertNotNull(certificate);
     }
     
     @Test
     public void testDirectReferenceSignature() throws Exception {
-        Document doc = readDocument("wsse-request-clean.xml");
-
-        WSS4JOutInterceptor ohandler = new WSS4JOutInterceptor();
-        PhaseInterceptor<SoapMessage> handler = ohandler.createEndingInterceptor();
-
-        SoapMessage msg = new SoapMessage(new MessageImpl());
-        Exchange ex = new ExchangeImpl();
-        ex.setInMessage(msg);
+        Map<String, String> outProperties = new HashMap<String, String>();
+        outProperties.put(WSHandlerConstants.ACTION, WSHandlerConstants.SIGNATURE);
+        outProperties.put(WSHandlerConstants.SIG_PROP_FILE, "outsecurity.properties");
+        outProperties.put(WSHandlerConstants.USER, "myalias");
+        outProperties.put(WSHandlerConstants.SIG_KEY_ID, "DirectReference");
+        outProperties.put("password", "myAliasPassword");
         
-        SOAPMessage saajMsg = MessageFactory.newInstance().createMessage();
-        SOAPPart part = saajMsg.getSOAPPart();
-        part.setContent(new DOMSource(doc));
-        saajMsg.saveChanges();
-
-        msg.setContent(SOAPMessage.class, saajMsg);
-
-        msg.put(WSHandlerConstants.ACTION, WSHandlerConstants.SIGNATURE);
-        msg.put(WSHandlerConstants.SIG_PROP_FILE, "META-INF/cxf/outsecurity.properties");
-        msg.put(WSHandlerConstants.USER, "myalias");
-        msg.put(WSHandlerConstants.SIG_KEY_ID, "DirectReference");
-        msg.put("password", "myAliasPassword");
-
-        handler.handleMessage(msg);
-
-        doc = part;
+        Map<String, String> inProperties = new HashMap<String, String>();
+        inProperties.put(WSHandlerConstants.ACTION, WSHandlerConstants.SIGNATURE);
+        inProperties.put(WSHandlerConstants.SIG_PROP_FILE, "insecurity.properties");
         
-        assertValid("//wsse:Security", doc);
-        // Check to see that the binary security token was inserted in the header
-        assertValid("//wsse:Security/wsse:BinarySecurityToken", doc);
-        assertValid("//wsse:Security/ds:Signature", doc);
+        List<String> xpaths = new ArrayList<String>();
+        xpaths.add("//wsse:Security");
+        xpaths.add("//wsse:Security/wsse:BinarySecurityToken");
+        xpaths.add("//wsse:Security/ds:Signature");
 
-        byte[] docbytes = getMessageBytes(doc);
-        XMLStreamReader reader = StaxUtils.createXMLStreamReader(new ByteArrayInputStream(docbytes));
-
-        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
-
-        dbf.setValidating(false);
-        dbf.setIgnoringComments(false);
-        dbf.setIgnoringElementContentWhitespace(true);
-        dbf.setNamespaceAware(true);
-
-        DocumentBuilder db = dbf.newDocumentBuilder();
-        db.setEntityResolver(new NullResolver());
-        doc = StaxUtils.read(db, reader, false);
-
-        WSS4JInInterceptor inHandler = new WSS4JInInterceptor();
-
-        SoapMessage inmsg = new SoapMessage(new MessageImpl());
-        ex.setInMessage(inmsg);
-        inmsg.setContent(SOAPMessage.class, saajMsg);
-
-        inHandler.setProperty(WSHandlerConstants.ACTION, WSHandlerConstants.SIGNATURE);
-        inHandler.setProperty(WSHandlerConstants.SIG_PROP_FILE, "META-INF/cxf/insecurity.properties");
-
-        inHandler.handleMessage(inmsg);
-        
-        WSSecurityEngineResult result = 
-            (WSSecurityEngineResult) inmsg.get(WSS4JInInterceptor.SIGNATURE_RESULT);
-        assertNotNull(result);
-        X509Certificate certificate = (X509Certificate)result
-            .get(WSSecurityEngineResult.TAG_X509_CERTIFICATE);
+        List<WSHandlerResult> handlerResults = 
+            getResults(makeInvocation(outProperties, xpaths, inProperties));
+        WSSecurityEngineResult actionResult =
+            WSSecurityUtil.fetchActionResult(handlerResults.get(0).getResults(), WSConstants.SIGN);
+         
+        X509Certificate certificate = 
+            (X509Certificate) actionResult.get(WSSecurityEngineResult.TAG_X509_CERTIFICATE);
         assertNotNull(certificate);
     }
     
     @Test
-    @SuppressWarnings("unchecked")
     public void testEncryption() throws Exception {
-        Document doc = readDocument("wsse-request-clean.xml");
-
-        WSS4JOutInterceptor ohandler = new WSS4JOutInterceptor();
-        PhaseInterceptor<SoapMessage> handler = ohandler.createEndingInterceptor();
-
-        SoapMessage msg = new SoapMessage(new MessageImpl());
-        Exchange ex = new ExchangeImpl();
-        ex.setInMessage(msg);
+        Map<String, String> outProperties = new HashMap<String, String>();
+        outProperties.put(WSHandlerConstants.ACTION, WSHandlerConstants.ENCRYPT);
+        outProperties.put(WSHandlerConstants.ENC_PROP_FILE, "outsecurity.properties");
+        outProperties.put(WSHandlerConstants.USER, "myalias");
+        outProperties.put("password", "myAliasPassword");
         
-        SOAPMessage saajMsg = MessageFactory.newInstance().createMessage();
-        SOAPPart part = saajMsg.getSOAPPart();
-        part.setContent(new DOMSource(doc));
-        saajMsg.saveChanges();
-
-        msg.setContent(SOAPMessage.class, saajMsg);
-        
-        msg.put(WSHandlerConstants.ACTION, WSHandlerConstants.ENCRYPT);
-        msg.put(WSHandlerConstants.SIG_PROP_FILE, "META-INF/cxf/outsecurity.properties");
-        msg.put(WSHandlerConstants.ENC_PROP_FILE, "META-INF/cxf/outsecurity.properties");
-        msg.put(WSHandlerConstants.USER, "myalias");
-        msg.put("password", "myAliasPassword");
-
-        handler.handleMessage(msg);
-        doc = part;
-
-        assertValid("//wsse:Security", doc);
-        assertValid("//s:Body/xenc:EncryptedData", doc);
-
-        byte[] docbytes = getMessageBytes(doc);
-        XMLStreamReader reader = StaxUtils.createXMLStreamReader(new ByteArrayInputStream(docbytes));
-
-        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
-
-        dbf.setValidating(false);
-        dbf.setIgnoringComments(false);
-        dbf.setIgnoringElementContentWhitespace(true);
-        dbf.setNamespaceAware(true);
-
-        DocumentBuilder db = dbf.newDocumentBuilder();
-        db.setEntityResolver(new NullResolver());
-        doc = StaxUtils.read(db, reader, false);
-
-        WSS4JInInterceptor inHandler = new WSS4JInInterceptor();
-
-        SoapMessage inmsg = new SoapMessage(new MessageImpl());
-        ex.setInMessage(inmsg);
-        inmsg.setContent(SOAPMessage.class, saajMsg);
-
-        inHandler.setProperty(WSHandlerConstants.ACTION, WSHandlerConstants.ENCRYPT);
-        inHandler.setProperty(WSHandlerConstants.DEC_PROP_FILE, "META-INF/cxf/insecurity.properties");
-        inHandler.setProperty(
+        Map<String, String> inProperties = new HashMap<String, String>();
+        inProperties.put(WSHandlerConstants.ACTION, WSHandlerConstants.ENCRYPT);
+        inProperties.put(WSHandlerConstants.DEC_PROP_FILE, "insecurity.properties");
+        inProperties.put(
             WSHandlerConstants.PW_CALLBACK_CLASS, 
             "org.apache.cxf.ws.security.wss4j.TestPwdCallback"
         );
+        
+        List<String> xpaths = new ArrayList<String>();
+        xpaths.add("//wsse:Security");
+        xpaths.add("//s:Body/xenc:EncryptedData");
+
+        List<WSHandlerResult> handlerResults = 
+            getResults(makeInvocation(outProperties, xpaths, inProperties));
 
-        inHandler.handleMessage(inmsg);
-        //
-        // Check that the EncryptedData is no longer there
-        //
-        assertInvalid("//s:Body/xenc:EncryptedData", saajMsg.getSOAPPart());
-        //
-        // There should be exactly 1 (WSS4J) HandlerResult
-        //
-        final java.util.List<WSHandlerResult> handlerResults = 
-            (java.util.List<WSHandlerResult>) inmsg.get(WSHandlerConstants.RECV_RESULTS);
         assertNotNull(handlerResults);
         assertSame(handlerResults.size(), 1);
         //
         // This should contain exactly 1 protection result
         //
-        final java.util.List<Object> protectionResults =
-            (java.util.List<Object>) handlerResults.get(0).getResults();
+        final java.util.List<WSSecurityEngineResult> protectionResults =
+            (java.util.List<WSSecurityEngineResult>) handlerResults.get(0).getResults();
         assertNotNull(protectionResults);
         assertSame(protectionResults.size(), 1);
         //
@@ -302,8 +185,7 @@ public class WSS4JInOutTest extends AbstractSecurityTest {
         final java.util.Map<String, Object> result =
             (java.util.Map<String, Object>) protectionResults.get(0);
         final java.util.List<WSDataRef> protectedElements =
-            (java.util.List<WSDataRef>) 
-                result.get(WSSecurityEngineResult.TAG_DATA_REF_URIS);
+            CastUtils.cast((List<?>)result.get(WSSecurityEngineResult.TAG_DATA_REF_URIS));
         assertNotNull(protectedElements);
         assertSame(protectedElements.size(), 1);
         assertEquals(
@@ -316,98 +198,51 @@ public class WSS4JInOutTest extends AbstractSecurityTest {
     }
     
     @Test
-    @SuppressWarnings("unchecked")
     public void testEncryptedUsernameToken() throws Exception {
-        Document doc = readDocument("wsse-request-clean.xml");
-
-        WSS4JOutInterceptor ohandler = new WSS4JOutInterceptor();
-        PhaseInterceptor<SoapMessage> handler = ohandler.createEndingInterceptor();
-
-        SoapMessage msg = new SoapMessage(new MessageImpl());
-        Exchange ex = new ExchangeImpl();
-        ex.setInMessage(msg);
-        
-        SOAPMessage saajMsg = MessageFactory.newInstance().createMessage();
-        SOAPPart part = saajMsg.getSOAPPart();
-        part.setContent(new DOMSource(doc));
-        saajMsg.saveChanges();
-
-        msg.setContent(SOAPMessage.class, saajMsg);
-        
-        msg.put(
-            WSHandlerConstants.ACTION, 
+        Map<String, String> outProperties = new HashMap<String, String>();
+        outProperties.put(
+            WSHandlerConstants.ACTION,
             WSHandlerConstants.USERNAME_TOKEN + " " + WSHandlerConstants.ENCRYPT
         );
-        msg.put(WSHandlerConstants.ENC_PROP_FILE, "META-INF/cxf/outsecurity.properties");
-        msg.put(WSHandlerConstants.USER, "alice");
-        msg.put("password", "alicePassword");
-        msg.put(WSHandlerConstants.ENCRYPTION_USER, "myalias");
-        msg.put(
+        outProperties.put(WSHandlerConstants.ENC_PROP_FILE, "outsecurity.properties");
+        outProperties.put(WSHandlerConstants.USER, "alice");
+        outProperties.put("password", "alicePassword");
+        outProperties.put(WSHandlerConstants.ENCRYPTION_USER, "myalias");
+        outProperties.put(
             WSHandlerConstants.ENCRYPTION_PARTS, 
             "{Content}{" + WSConstants.WSSE_NS + "}UsernameToken"
         );
-
-        handler.handleMessage(msg);
-        doc = part;
-
-        assertValid("//wsse:Security", doc);
-
-        byte[] docbytes = getMessageBytes(doc);
-        XMLStreamReader reader = StaxUtils.createXMLStreamReader(new ByteArrayInputStream(docbytes));
-
-        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
-
-        dbf.setValidating(false);
-        dbf.setIgnoringComments(false);
-        dbf.setIgnoringElementContentWhitespace(true);
-        dbf.setNamespaceAware(true);
-
-        DocumentBuilder db = dbf.newDocumentBuilder();
-        db.setEntityResolver(new NullResolver());
-        doc = StaxUtils.read(db, reader, false);
-
-        WSS4JInInterceptor inHandler = new WSS4JInInterceptor();
-
-        SoapMessage inmsg = new SoapMessage(new MessageImpl());
-        ex.setInMessage(inmsg);
-        inmsg.setContent(SOAPMessage.class, saajMsg);
-
-        inHandler.setProperty(
+        
+        Map<String, String> inProperties = new HashMap<String, String>();
+        inProperties.put(
             WSHandlerConstants.ACTION, 
             WSHandlerConstants.USERNAME_TOKEN + " " + WSHandlerConstants.ENCRYPT
         );
-        inHandler.setProperty(WSHandlerConstants.DEC_PROP_FILE, "META-INF/cxf/insecurity.properties");
-        inHandler.setProperty(
+        inProperties.put(WSHandlerConstants.DEC_PROP_FILE, "insecurity.properties");
+        inProperties.put(
             WSHandlerConstants.PW_CALLBACK_CLASS, 
             "org.apache.cxf.ws.security.wss4j.TestPwdCallback"
         );
+        
+        List<String> xpaths = new ArrayList<String>();
+        xpaths.add("//wsse:Security");
+
+        SoapMessage inmsg = makeInvocation(outProperties, xpaths, inProperties);
+        List<WSHandlerResult> handlerResults = getResults(inmsg);
 
-        inHandler.handleMessage(inmsg);
-        //
-        // Check that the EncryptedData is no longer there
-        //
-        assertInvalid("//s:Body/xenc:EncryptedData", saajMsg.getSOAPPart());
-        //
-        // There should be exactly 1 (WSS4J) HandlerResult
-        //
-        final java.util.List<WSHandlerResult> handlerResults = 
-            (java.util.List<WSHandlerResult>) inmsg.get(WSHandlerConstants.RECV_RESULTS);
         assertNotNull(handlerResults);
         assertSame(handlerResults.size(), 1);
         
         //
         // This should contain exactly 2 protection results
         //
-        final java.util.List<Object> protectionResults =
-            (java.util.List<Object>) handlerResults.get(0).getResults();
+        final java.util.List<WSSecurityEngineResult> protectionResults =
+            (java.util.List<WSSecurityEngineResult>) handlerResults.get(0).getResults();
         assertNotNull(protectionResults);
         assertSame(protectionResults.size(), 2);
         
-        WSSecurityEngineResult wsResult1 = (WSSecurityEngineResult)protectionResults.get(0);
-        WSSecurityEngineResult wsResult2 = (WSSecurityEngineResult)protectionResults.get(1);
-
-        final Principal p1 = (Principal)wsResult1.get(WSSecurityEngineResult.TAG_PRINCIPAL);
-        final Principal p2 = (Principal)wsResult2.get(WSSecurityEngineResult.TAG_PRINCIPAL);
+        final Principal p1 = (Principal)protectionResults.get(0).get(WSSecurityEngineResult.TAG_PRINCIPAL);
+        final Principal p2 = (Principal)protectionResults.get(1).get(WSSecurityEngineResult.TAG_PRINCIPAL);
         assertTrue(p1 instanceof WSUsernameTokenPrincipal || p2 instanceof WSUsernameTokenPrincipal);
         
         Principal utPrincipal = p1 instanceof WSUsernameTokenPrincipal ? p1 : p2;
@@ -416,6 +251,45 @@ public class WSS4JInOutTest extends AbstractSecurityTest {
         assertSame(secContextPrincipal, utPrincipal);
     }
     
+    @Test
+    public void testUsernameToken() throws Exception {
+        Map<String, String> outProperties = new HashMap<String, String>();
+        outProperties.put(WSHandlerConstants.ACTION, WSHandlerConstants.USERNAME_TOKEN);
+        outProperties.put(WSHandlerConstants.PASSWORD_TYPE, WSConstants.PW_TEXT);
+        outProperties.put(WSHandlerConstants.USER, "alice");
+        outProperties.put("password", "alicePassword");
+        
+        Map<String, String> inProperties = new HashMap<String, String>();
+        inProperties.put(WSHandlerConstants.ACTION, WSHandlerConstants.USERNAME_TOKEN);
+        inProperties.put(WSHandlerConstants.PASSWORD_TYPE, WSConstants.PW_DIGEST);
+        inProperties.put(WSHandlerConstants.PASSWORD_TYPE_STRICT, "false");
+        inProperties.put(
+            WSHandlerConstants.PW_CALLBACK_CLASS, 
+            "org.apache.cxf.ws.security.wss4j.TestPwdCallback"
+        );
+        
+        List<String> xpaths = new ArrayList<String>();
+        xpaths.add("//wsse:Security");
+
+        //
+        // This should pass, as even though passwordType is set to digest, we are 
+        // overriding the default handler behaviour of requiring a strict password
+        // type
+        //
+        makeInvocation(outProperties, xpaths, inProperties);
+        
+        //
+        // This should fail, as we are requiring a digest password type
+        //
+        inProperties.put(WSHandlerConstants.PASSWORD_TYPE_STRICT, "true");
+        try {
+            makeInvocation(outProperties, xpaths, inProperties);
+            fail("Failure expected on the wrong password type");
+        } catch (org.apache.cxf.interceptor.Fault fault) {
+            // expected
+        }
+    }
+    
     @Test
     public void testCustomProcessor() throws Exception {
         Document doc = readDocument("wsse-request-clean.xml");
@@ -435,7 +309,7 @@ public class WSS4JInOutTest extends AbstractSecurityTest {
         msg.setContent(SOAPMessage.class, saajMsg);
 
         msg.put(WSHandlerConstants.ACTION, WSHandlerConstants.SIGNATURE);
-        msg.put(WSHandlerConstants.SIG_PROP_FILE, "META-INF/cxf/outsecurity.properties");
+        msg.put(WSHandlerConstants.SIG_PROP_FILE, "outsecurity.properties");
         msg.put(WSHandlerConstants.USER, "myalias");
         msg.put("password", "myAliasPassword");
 
@@ -480,7 +354,6 @@ public class WSS4JInOutTest extends AbstractSecurityTest {
         assertNull(result);
     }
     
-    
     @Test
     public void testCustomProcessorObject() throws Exception {
         Document doc = readDocument("wsse-request-clean.xml");
@@ -500,7 +373,7 @@ public class WSS4JInOutTest extends AbstractSecurityTest {
         msg.setContent(SOAPMessage.class, saajMsg);
 
         msg.put(WSHandlerConstants.ACTION, WSHandlerConstants.SIGNATURE);
-        msg.put(WSHandlerConstants.SIG_PROP_FILE, "META-INF/cxf/outsecurity.properties");
+        msg.put(WSHandlerConstants.SIG_PROP_FILE, "outsecurity.properties");
         msg.put(WSHandlerConstants.USER, "myalias");
         msg.put("password", "myAliasPassword");
 
@@ -532,7 +405,7 @@ public class WSS4JInOutTest extends AbstractSecurityTest {
                 WSConstants.SIG_NS,
                 WSConstants.SIG_LN
             ),
-            new CustomProcessor()
+            CustomProcessor.class
         );
         properties.put(
             WSS4JInInterceptor.PROCESSOR_MAP,
@@ -559,71 +432,71 @@ public class WSS4JInOutTest extends AbstractSecurityTest {
     
     @Test
     public void testPKIPath() throws Exception {
-        Document doc = readDocument("wsse-request-clean.xml");
-
-        WSS4JOutInterceptor ohandler = new WSS4JOutInterceptor();
-        PhaseInterceptor<SoapMessage> handler = ohandler.createEndingInterceptor();
-
-        SoapMessage msg = new SoapMessage(new MessageImpl());
-        Exchange ex = new ExchangeImpl();
-        ex.setInMessage(msg);
+        Map<String, String> outProperties = new HashMap<String, String>();
+        outProperties.put(WSHandlerConstants.ACTION, WSHandlerConstants.SIGNATURE);
+        outProperties.put(WSHandlerConstants.USER, "alice");
+        outProperties.put(WSHandlerConstants.SIG_PROP_FILE, "alice.properties");
+        outProperties.put(
+            WSHandlerConstants.PW_CALLBACK_CLASS, KeystorePasswordCallback.class.getName()
+        );
+        outProperties.put(WSHandlerConstants.SIG_KEY_ID, "DirectReference");
+        outProperties.put(WSHandlerConstants.USE_SINGLE_CERTIFICATE, "false");
         
-        SOAPMessage saajMsg = MessageFactory.newInstance().createMessage();
-        SOAPPart part = saajMsg.getSOAPPart();
-        part.setContent(new DOMSource(doc));
-        saajMsg.saveChanges();
-
-        msg.setContent(SOAPMessage.class, saajMsg);
-
-        msg.put(WSHandlerConstants.ACTION, WSHandlerConstants.SIGNATURE);
-        msg.put(WSHandlerConstants.SIG_PROP_FILE, "META-INF/cxf/alice.properties");
-        msg.put(WSHandlerConstants.USER, "alice");
-        msg.put(WSHandlerConstants.PW_CALLBACK_CLASS, KeystorePasswordCallback.class.getName());
-        msg.put(WSHandlerConstants.SIG_KEY_ID, "DirectReference");
-        msg.put(WSHandlerConstants.USE_SINGLE_CERTIFICATE, "false");
-
-        handler.handleMessage(msg);
-
-        doc = part;
+        Map<String, String> inProperties = new HashMap<String, String>();
+        inProperties.put(WSHandlerConstants.ACTION, WSHandlerConstants.SIGNATURE);
+        inProperties.put(WSHandlerConstants.SIG_PROP_FILE, "cxfca.properties");
         
-        assertValid("//wsse:Security", doc);
-        assertValid("//wsse:Security/ds:Signature", doc);
+        List<String> xpaths = new ArrayList<String>();
+        xpaths.add("//wsse:Security");
+        xpaths.add("//wsse:Security/ds:Signature");
 
-        byte[] docbytes = getMessageBytes(doc);
-
-        XMLStreamReader reader = StaxUtils.createXMLStreamReader(new ByteArrayInputStream(docbytes));
-
-        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
-
-        dbf.setValidating(false);
-        dbf.setIgnoringComments(false);
-        dbf.setIgnoringElementContentWhitespace(true);
-        dbf.setNamespaceAware(true);
-
-        DocumentBuilder db = dbf.newDocumentBuilder();
-        db.setEntityResolver(new NullResolver());
-        doc = StaxUtils.read(db, reader, false);
-
-        WSS4JInInterceptor inHandler = new WSS4JInInterceptor();
-
-        SoapMessage inmsg = new SoapMessage(new MessageImpl());
-        ex.setInMessage(inmsg);
-        inmsg.setContent(SOAPMessage.class, saajMsg);
-
-        inHandler.setProperty(WSHandlerConstants.ACTION, WSHandlerConstants.SIGNATURE);
-        inHandler.setProperty(WSHandlerConstants.SIG_PROP_FILE, "META-INF/cxf/cxfca.properties");
-
-        inHandler.handleMessage(inmsg);
-        
-        WSSecurityEngineResult result = 
-            (WSSecurityEngineResult) inmsg.get(WSS4JInInterceptor.SIGNATURE_RESULT);
-        assertNotNull(result);
-        X509Certificate[] certificates = (X509Certificate[])result
-            .get(WSSecurityEngineResult.TAG_X509_CERTIFICATES);
+        List<WSHandlerResult> handlerResults = 
+            getResults(makeInvocation(outProperties, xpaths, inProperties));
+        WSSecurityEngineResult actionResult =
+            WSSecurityUtil.fetchActionResult(handlerResults.get(0).getResults(), WSConstants.SIGN);
+         
+        X509Certificate[] certificates = 
+            (X509Certificate[]) actionResult.get(WSSecurityEngineResult.TAG_X509_CERTIFICATES);
         assertNotNull(certificates);
         assertEquals(certificates.length, 2);
     }
     
+    @Test
+    public void testUsernameTokenSignature() throws Exception {
+        Map<String, String> outProperties = new HashMap<String, String>();
+        outProperties.put(
+            WSHandlerConstants.ACTION, 
+            WSHandlerConstants.USERNAME_TOKEN + " " + WSHandlerConstants.SIGNATURE);
+        outProperties.put(WSHandlerConstants.PASSWORD_TYPE, WSConstants.PW_TEXT);
+        outProperties.put(WSHandlerConstants.USER, "alice");
+        
+        outProperties.put(WSHandlerConstants.SIG_PROP_FILE, "outsecurity.properties");
+        outProperties.put(WSHandlerConstants.SIGNATURE_USER, "myalias");
+        outProperties.put(
+            WSHandlerConstants.PW_CALLBACK_CLASS, 
+            "org.apache.cxf.ws.security.wss4j.TestPwdCallback"
+        );
+        
+        Map<String, String> inProperties = new HashMap<String, String>();
+        inProperties.put(
+            WSHandlerConstants.ACTION, 
+            WSHandlerConstants.USERNAME_TOKEN + " " + WSHandlerConstants.SIGNATURE
+        );
+        inProperties.put(WSHandlerConstants.PASSWORD_TYPE, WSConstants.PW_TEXT);
+        inProperties.put(
+            WSHandlerConstants.PW_CALLBACK_CLASS, 
+            "org.apache.cxf.ws.security.wss4j.TestPwdCallback"
+        );
+        inProperties.put(WSHandlerConstants.SIG_PROP_FILE, "insecurity.properties");
+        
+        List<String> xpaths = new ArrayList<String>();
+        xpaths.add("//wsse:Security");
+        xpaths.add("//wsse:Security/ds:Signature");
+        xpaths.add("//wsse:Security/wsse:UsernameToken");
+
+        makeInvocation(outProperties, xpaths, inProperties);
+    }
+    
     
     private byte[] getMessageBytes(Document doc) throws Exception {
         ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
@@ -652,6 +525,73 @@ public class WSS4JInOutTest extends AbstractSecurityTest {
         return ret;
     }
     
+    private List<WSHandlerResult> getResults(SoapMessage inmsg) {
+        final List<WSHandlerResult> handlerResults = 
+            CastUtils.cast((List<?>)inmsg.get(WSHandlerConstants.RECV_RESULTS));
+        return handlerResults;
+    }
+    
+    private SoapMessage makeInvocation(
+        Map<String, String> outProperties,
+        List<String> xpaths,
+        Map<String, String> inProperties
+    ) throws Exception {
+        Document doc = readDocument("wsse-request-clean.xml");
+
+        WSS4JOutInterceptor ohandler = new WSS4JOutInterceptor();
+        PhaseInterceptor<SoapMessage> handler = ohandler.createEndingInterceptor();
+
+        SoapMessage msg = new SoapMessage(new MessageImpl());
+        Exchange ex = new ExchangeImpl();
+        ex.setInMessage(msg);
+
+        SOAPMessage saajMsg = MessageFactory.newInstance().createMessage();
+        SOAPPart part = saajMsg.getSOAPPart();
+        part.setContent(new DOMSource(doc));
+        saajMsg.saveChanges();
+
+        msg.setContent(SOAPMessage.class, saajMsg);
+
+        for (String key : outProperties.keySet()) {
+            msg.put(key, outProperties.get(key));
+        }
+
+        handler.handleMessage(msg);
+
+        doc = part;
+
+        for (String xpath : xpaths) {
+            assertValid(xpath, doc);
+        }
+
+        byte[] docbytes = getMessageBytes(doc);
+        XMLStreamReader reader = StaxUtils.createXMLStreamReader(new ByteArrayInputStream(docbytes));
+
+        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
+
+        dbf.setValidating(false);
+        dbf.setIgnoringComments(false);
+        dbf.setIgnoringElementContentWhitespace(true);
+        dbf.setNamespaceAware(true);
+
+        DocumentBuilder db = dbf.newDocumentBuilder();
+        db.setEntityResolver(new NullResolver());
+        doc = StaxUtils.read(db, reader, false);
+
+        WSS4JInInterceptor inHandler = new WSS4JInInterceptor();
+
+        SoapMessage inmsg = new SoapMessage(new MessageImpl());
+        ex.setInMessage(inmsg);
+        inmsg.setContent(SOAPMessage.class, saajMsg);
+
+        for (String key : inProperties.keySet()) {
+            inHandler.setProperty(key, inProperties.get(key));
+        }
+
+        inHandler.handleMessage(inmsg);
+
+        return inmsg;
+    }
     
     // FOR DEBUGGING ONLY
     /*private*/ static String serialize(Document doc) {
