@@ -75,12 +75,9 @@ import javax.xml.transform.dom.DOMSource;
 import org.apache.cxf.annotations.GZIP;
 import org.apache.cxf.common.util.ProxyHelper;
 import org.apache.cxf.helpers.XMLUtils;
-import org.apache.cxf.jaxrs.ext.Nullable;
 import org.apache.cxf.jaxrs.ext.Oneway;
 import org.apache.cxf.jaxrs.ext.search.SearchCondition;
 import org.apache.cxf.jaxrs.ext.search.SearchContext;
-import org.apache.cxf.jaxrs.ext.xml.XMLInstruction;
-import org.apache.cxf.jaxrs.ext.xml.XSISchemaLocation;
 import org.apache.cxf.phase.PhaseInterceptorChain;
 
 @Path("/bookstore")
@@ -92,9 +89,6 @@ public class BookStore {
     private long bookId = 123;
     private long cdId = 123;
     
-    private String defaultName;
-    private long defaultId;
-    
     private String currentBookId;
     @PathParam("CDId")
     private String currentCdId;
@@ -118,19 +112,6 @@ public class BookStore {
     public void preDestroy() {
         //System.out.println("PreDestroy called");
     }
-
-    @GET
-    @Path("allCharsButA-B/:@!$&'()*+,;=-._~")
-    public Book getWithComplexPath() {
-        return new Book("Encoded Path", 125L);
-    }
-    
-    @GET
-    @Path("/default")
-    @Produces("application/xml")
-    public Book getDefaultBook() {
-        return new Book(defaultName, defaultId);
-    }
     
     @GET
     @Path("/books/colon/{a}:{b}:{c}")
@@ -217,8 +198,6 @@ public class BookStore {
     @GET
     @Path("name-in-query")
     @Produces("application/xml")
-    @XMLInstruction("<!DOCTYPE Something SYSTEM 'my.dtd'><?xmlstylesheet href='common.css'?>")
-    @XSISchemaLocation("book.xsd")
     public Book getBookFromQuery(@QueryParam("name") String name) {
         return new Book(name, 321L);
     }
@@ -314,16 +293,6 @@ public class BookStore {
         return bs;
     }
     
-    @POST
-    @Path("/collectionBook")
-    @Produces({"application/xml", "application/json" })
-    @Consumes({"application/xml", "application/json" })
-    public List<Book> postBookGetCollection(Book book) throws Exception {
-        List<Book> list = new ArrayList<Book>();
-        list.add(book);
-        return list;
-    }
-    
     @POST
     @Path("/collections3")
     @Produces({"application/xml", "application/json" })
@@ -393,14 +362,6 @@ public class BookStore {
         return doGetBook(s1 + s2);
     }
     
-    @GET
-    @Path("/segment/matrix-middle")
-    public Book getBookByMatrixParamsMiddle(@MatrixParam("first") String s1,
-                                      @MatrixParam("second") String s2) throws Exception {
-        
-        return doGetBook(s1 + s2);
-    }
-    
     @GET
     @Path("/segment/matrix-list")
     public Book getBookByMatrixListParams(@MatrixParam("first") List<String> list) throws Exception {
@@ -677,11 +638,6 @@ public class BookStore {
         currentBookId = id;
     }
     
-    public void setDefaultNameAndId(String name, long id) {
-        defaultName = name;
-        defaultId = id;
-    }
-    
     @GET
     @Path("/books/{bookId}/")
     @Produces("application/json;q=0.9")
@@ -753,17 +709,6 @@ public class BookStore {
         }
     }
 
-    @POST
-    @Path("/books/null")
-    @Produces("application/xml")
-    @Consumes("application/xml")
-    public Book handleNullBook(@Nullable Book book) {
-        if (book != null) {
-            throw new WebApplicationException(400);
-        }
-        return new Book("Default Book", 222L);
-    }
-    
     @POST
     @Path("/books")
     @Produces("text/xml")
@@ -1012,14 +957,7 @@ public class BookStore {
         return new BookSubresourceImpl();
     }
     
-    @GET
-    @Path("/reset")
-    public final String init() {
-        books.clear();
-        cds.clear();
-        bookId = 123;
-        cdId = 123;        
-        
+    final void init() {
         Book book = new Book();
         book.setId(bookId);
         book.setName("CXF in Action");
@@ -1033,11 +971,10 @@ public class BookStore {
         cd1.setId(++cdId);
         cd1.setName("BICYCLE RACE");
         cds.put(cd1.getId(), cd1);
-        return "OK";
     }
     
     @XmlJavaTypeAdapter(BookInfoAdapter2.class)
-    interface BookInfoInterface {
+    static interface BookInfoInterface {
         String getName();
         
         long getId();
