@@ -45,13 +45,17 @@ import org.apache.commons.httpclient.methods.FileRequestEntity;
 import org.apache.commons.httpclient.methods.PostMethod;
 import org.apache.commons.httpclient.methods.RequestEntity;
 import org.apache.cxf.Bus;
+import org.apache.cxf.endpoint.Client;
 import org.apache.cxf.feature.AbstractFeature;
+import org.apache.cxf.frontend.ClientProxy;
 import org.apache.cxf.helpers.IOUtils;
 import org.apache.cxf.interceptor.FIStaxInInterceptor;
 import org.apache.cxf.interceptor.FIStaxOutInterceptor;
 import org.apache.cxf.interceptor.Fault;
 import org.apache.cxf.interceptor.Interceptor;
 import org.apache.cxf.interceptor.InterceptorProvider;
+import org.apache.cxf.interceptor.transform.TransformInInterceptor;
+import org.apache.cxf.interceptor.transform.TransformOutInterceptor;
 import org.apache.cxf.io.CachedOutputStream;
 import org.apache.cxf.jaxrs.client.ClientConfiguration;
 import org.apache.cxf.jaxrs.client.JAXRSClientFactory;
@@ -150,6 +154,56 @@ public class JAXRSSoapBookTest extends AbstractBusClientServerTestBase {
                 
     }
     
+    @Test
+    public void testGetBookTransform() throws Exception {
+        
+        String address = "http://localhost:" + PORT 
+                         + "/test/v1/rest-transform/bookstore/books/123";
+        WebClient client = WebClient.create(address);
+        Response r = client.get();
+        String str = getStringFromInputStream((InputStream)r.getEntity());
+        assertTrue(str.contains("TheBook"));
+    }
+    
+    @Test
+    public void testPostBookTransform() throws Exception {
+           
+        String address = "http://localhost:" + PORT 
+                         + "/test/v1/rest-transform/bookstore/books";
+        
+        TransformOutInterceptor out =  new TransformOutInterceptor();
+        out.setOutTransformElements(
+            Collections.singletonMap("{http://www.example.org/books}*", 
+                                     "{http://www.example.org/super-books}*"));
+        
+        TransformInInterceptor in =  new TransformInInterceptor();
+        Map<String, String> map = new HashMap<String, String>();
+
+        // If Book2 didn't have {http://www.example.org/books}Book
+        // then we'd just do '"*" : "{http://www.example.org/books}*'
+        // but given that we have TheBook being returned, we need
+        map.put("TheBook", "{http://www.example.org/books}Book");
+        map.put("id", "{http://www.example.org/books}id");
+        in.setInTransformElements(map);
+        
+        WebClient client = WebClient.create(address);
+        WebClient.getConfig(client).getInInterceptors().add(in);
+        WebClient.getConfig(client).getOutInterceptors().add(out);
+        Book2 book = client.accept("text/xml").post(new Book2(), Book2.class);
+        assertEquals(124L, book.getId());
+    }
+    
+    @Test
+    public void testPostBookTransformV2() throws Exception {
+           
+        String address = "http://localhost:" + PORT 
+                         + "/test/v2/rest-transform/bookstore/books";
+        WebClient client = WebClient.create(address);
+        Book book = client.accept("text/xml").post(new Book(), Book.class);
+        assertEquals(124L, book.getId());
+    }
+    
+    
     @Test
     public void testGetBookFastinfoset() throws Exception {
         
@@ -231,7 +285,7 @@ public class JAXRSSoapBookTest extends AbstractBusClientServerTestBase {
         Book b = new Book("CXF", 1L);
         
         // Just to make sure it is enforced
-        Map<String, Object> props = WebClient.getConfig(client).getResponseContext();
+        Map<String, Object> props = WebClient.getConfig(client).getRequestContext();
         props.put(FIStaxOutInterceptor.FI_ENABLED, Boolean.TRUE);
         
         Book b2 = client.addFastinfoBook(b);
@@ -632,6 +686,52 @@ public class JAXRSSoapBookTest extends AbstractBusClientServerTestBase {
         assertEquals("id is wrong", book.getId(), 123);
     }
     
+    @Test
+    public void testGetUnqualifiedBookSoap() throws Exception {
+        
+        String wsdlAddress =
+            "http://localhost:" + PORT + "/test/services/soap-transform/bookservice?wsdl";
+        BookSoapService service = 
+            new BookSoapService(new URL(wsdlAddress), 
+                                new QName("http://books.com", "BookService"));
+        BookStoreJaxrsJaxws store = service.getBookPort();
+        
+        TransformOutInterceptor out =  new TransformOutInterceptor();
+        Map<String, String> mapOut = new HashMap<String, String>();
+        // Book content (id, name) is unqualified, thus the following works
+        // because JAXB will report 
+        // - {http://jaxws.jaxrs.systest.cxf.apache.org/}Book
+        // - id
+        // - name
+        // and only the qualified top-level Book tag gets matched by the following
+        // mapping
+        mapOut.put("{http://jaxws.jaxrs.systest.cxf.apache.org/}*", "*");
+        out.setOutTransformElements(mapOut);
+        
+        TransformInInterceptor in =  new TransformInInterceptor();
+        Map<String, String> mapIn = new HashMap<String, String>();
+        // mapIn.put("*", "{http://jaxws.jaxrs.systest.cxf.apache.org/}*");
+        // won't work for a case where a totally unqualified getBookResponse needs to be
+        // qualified such that only the top-level getBookResponse is processed because of '*'.
+        // Such a mapping would work nicely if we had say a package-info making both
+        // Book id & name qualified; otherwise we need to choose what tag we need to qualify 
+        
+        // mapIn.put("*", "{http://jaxws.jaxrs.systest.cxf.apache.org/}*");
+        // works too if the schema validation is disabled
+        
+        mapIn.put("getBookResponse", "{http://jaxws.jaxrs.systest.cxf.apache.org/}getBookResponse");
+        in.setInTransformElements(mapIn);
+        
+        Client cl = ClientProxy.getClient(store);
+        ((HTTPConduit)cl.getConduit()).getClient().setReceiveTimeout(10000000);
+        cl.getInInterceptors().add(in);
+        cl.getOutInterceptors().add(out);
+        
+        Book book = store.getBook(new Long(123));
+        assertEquals("id is wrong", book.getId(), 123);
+        
+    }
+    
     @Test
     public void testServiceListingsAndWadl() throws Exception {
         String listings = 
@@ -652,7 +752,7 @@ public class JAXRSSoapBookTest extends AbstractBusClientServerTestBase {
         WebClient webClient = 
             WebClient.create("http://localhost:" + PORT + "/test/services/rest3?_wadl&type=xml");
         WebClient.getConfig(webClient).getHttpConduit().getClient().setReceiveTimeout(1000000);
-        assertEquals(401, webClient.get().getStatus());
+        assertEquals(404, webClient.get().getStatus());
     }
     
     @Test
@@ -663,7 +763,7 @@ public class JAXRSSoapBookTest extends AbstractBusClientServerTestBase {
         bean.setResourceClass(BookStoreJaxrsJaxws.class);
         TestFeature testFeature = new TestFeature();
         List<AbstractFeature> features = new ArrayList<AbstractFeature>();
-        features.add((AbstractFeature)testFeature);
+        features.add(testFeature);
         bean.setFeatures(features);
         BookStoreJaxrsJaxws proxy = (BookStoreJaxrsJaxws)bean.create();
         Book b = proxy.getBook(new Long("123"));
@@ -703,7 +803,8 @@ public class JAXRSSoapBookTest extends AbstractBusClientServerTestBase {
         } catch (Exception e) {
             assertTrue("Out Interceptor not invoked", testFeature.handleMessageOnOutInterceptorCalled());
             assertTrue("In Interceptor not invoked", !testFeature.handleMessageOnInInterceptorCalled());
-            assertTrue("Wrong exception caught", "fault from bad interceptor".equals(e.getMessage()));
+            assertTrue("Wrong exception caught", 
+                       "fault from bad interceptor".equals(e.getCause().getMessage()));
             assertTrue("Client In Fault In Interceptor was invoked", 
                     !testFeature.faultInInterceptorCalled());
         }
