@@ -22,6 +22,8 @@ package org.apache.cxf.systest.dispatch;
 import java.io.InputStream;
 import java.net.SocketTimeoutException;
 import java.net.URL;
+import java.util.HashMap;
+import java.util.Map;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Future;
 import java.util.concurrent.TimeUnit;
@@ -35,14 +37,19 @@ import javax.xml.transform.dom.DOMSource;
 import javax.xml.transform.sax.SAXSource;
 import javax.xml.transform.stream.StreamSource;
 import javax.xml.ws.AsyncHandler;
+import javax.xml.ws.BindingProvider;
 import javax.xml.ws.Dispatch;
 import javax.xml.ws.Endpoint;
 import javax.xml.ws.Response;
 import javax.xml.ws.Service;
 import javax.xml.ws.WebServiceException;
+import javax.xml.ws.soap.SOAPBinding;
 import javax.xml.ws.soap.SOAPFaultException;
+import javax.xml.ws.wsaddressing.W3CEndpointReference;
+import javax.xml.ws.wsaddressing.W3CEndpointReferenceBuilder;
 
 import org.w3c.dom.Document;
+import org.w3c.dom.Element;
 import org.w3c.dom.Node;
 
 import org.xml.sax.InputSource;
@@ -57,6 +64,7 @@ import org.apache.cxf.interceptor.LoggingOutInterceptor;
 import org.apache.cxf.jaxws.DispatchImpl;
 import org.apache.cxf.testutil.common.AbstractBusClientServerTestBase;
 import org.apache.cxf.testutil.common.AbstractBusTestServerBase;
+import org.apache.cxf.testutil.common.TestUtil;
 import org.apache.cxf.transport.http.HTTPConduit;
 import org.apache.cxf.transports.http.configuration.HTTPClientPolicy;
 import org.apache.hello_world_soap_http.BadRecordLitFault;
@@ -68,19 +76,33 @@ import org.apache.hello_world_soap_http.types.GreetMeResponse;
 import org.junit.BeforeClass;
 import org.junit.Test;
 
+
 public class DispatchClientServerTest extends AbstractBusClientServerTestBase {
 
-    private final QName serviceName = new QName("http://apache.org/hello_world_soap_http",
-                                                "SOAPDispatchService");
-    private final QName portName = new QName("http://apache.org/hello_world_soap_http", "SoapDispatchPort");
+    private static final QName SERVICE_NAME 
+        = new QName("http://apache.org/hello_world_soap_http", "SOAPDispatchService");
+    private static final QName PORT_NAME 
+        = new QName("http://apache.org/hello_world_soap_http", "SoapDispatchPort");
 
+    private static String greeterPort = TestUtil.getPortNumber(DispatchClientServerTest.class); 
+    
     public static class Server extends AbstractBusTestServerBase {        
 
         protected void run() {
             Object implementor = new GreeterImpl();
-            String address = "http://localhost:9006/SOAPDispatchService/SoapDispatchPort";
-            Endpoint.publish(address, implementor);
-
+            String address = "http://localhost:"
+                + TestUtil.getPortNumber(DispatchClientServerTest.class)
+                + "/SOAPDispatchService/SoapDispatchPort";
+            Endpoint ep = Endpoint.create(SOAPBinding.SOAP11HTTP_BINDING, implementor);
+            
+            Map<String, Object> properties = new HashMap<String, Object>();
+            Map<String, String> nsMap = new HashMap<String, String>();
+            nsMap.put("gmns", "http://apache.org/hello_world_soap_http/types");
+            properties.put("soap.env.ns.map", nsMap);
+            properties.put("disable.outputstream.optimization", "true");
+            
+            ep.setProperties(properties);
+            ep.publish(address);
         }
 
         public static void main(String[] args) {
@@ -135,6 +157,10 @@ public class DispatchClientServerTest extends AbstractBusClientServerTestBase {
 
         Dispatch<SOAPMessage> disp = service
             .createDispatch(otherPortName, SOAPMessage.class, Service.Mode.MESSAGE);
+        disp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY,
+                                     "http://localhost:" 
+                                     + TestUtil.getPortNumber("fake-port")
+                                     + "/SOAPDispatchService/SoapDispatchPort");
         
         DispatchImpl dispImpl = (DispatchImpl)disp;
         HTTPConduit cond = (HTTPConduit)dispImpl.getClient().getConduit();
@@ -151,24 +177,67 @@ public class DispatchClientServerTest extends AbstractBusClientServerTestBase {
             fail("should not be a SOAPFaultException");
         } catch (WebServiceException ex) {
             //expected
-            assertTrue(ex.getCause().getClass().getName(), 
-                       ex.getCause() instanceof java.net.ConnectException);
+            assertTrue(ex.getCause().getClass().getName(),
+                       ex.getCause() instanceof java.net.ConnectException
+                       || ex.getCause() instanceof java.net.SocketTimeoutException);
         }
         
     }
+    @Test
+    public void test404() throws Exception {
+        //CXF-2384
+        URL wsdl = getClass().getResource("/wsdl/hello_world.wsdl");
+        assertNotNull(wsdl);
 
+        //pick one of the other service/ports that would have an address
+        //without a service running
+        QName otherServiceName = new QName("http://apache.org/hello_world_soap_http",
+                "SOAPProviderService");
+        QName otherPortName = new QName("http://apache.org/hello_world_soap_http", "SoapProviderPort");
+        
+        
+        SOAPService service = new SOAPService(wsdl, otherServiceName);
+        assertNotNull(service);
+
+        Dispatch<SOAPMessage> disp = service
+            .createDispatch(otherPortName, SOAPMessage.class, Service.Mode.MESSAGE);
+        disp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY,
+                                     "http://localhost:" 
+                                     + greeterPort
+                                     + "/SomePlaceWithNoServiceRunning/SoapDispatchPort");
+        
+        InputStream is = getClass().getResourceAsStream("resources/GreetMeDocLiteralReq.xml");
+        SOAPMessage soapReqMsg = MessageFactory.newInstance().createMessage(null, is);
+        assertNotNull(soapReqMsg);
+        
+        try {
+            disp.invoke(soapReqMsg);
+            fail("Should have faulted");
+        } catch (SOAPFaultException ex) {
+            fail("should not be a SOAPFaultException");
+        } catch (WebServiceException ex) {
+            //expected
+            assertTrue(ex.getCause().getClass().getName(),
+                       ex.getCause() instanceof java.io.IOException);
+        }
+        
+    }
     @Test
     public void testSOAPMessage() throws Exception {
 
         URL wsdl = getClass().getResource("/wsdl/hello_world.wsdl");
         assertNotNull(wsdl);
 
-        SOAPService service = new SOAPService(wsdl, serviceName);
+        SOAPService service = new SOAPService(wsdl, SERVICE_NAME);
         assertNotNull(service);
 
         Dispatch<SOAPMessage> disp = service
-            .createDispatch(portName, SOAPMessage.class, Service.Mode.MESSAGE);
-
+            .createDispatch(PORT_NAME, SOAPMessage.class, Service.Mode.MESSAGE);
+        disp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY,
+                                     "http://localhost:" 
+                                     + greeterPort
+                                     + "/SOAPDispatchService/SoapDispatchPort");
+        
         // Test request-response
         InputStream is = getClass().getResourceAsStream("resources/GreetMeDocLiteralReq.xml");
         SOAPMessage soapReqMsg = MessageFactory.newInstance().createMessage(null, is);
@@ -213,6 +282,35 @@ public class DispatchClientServerTest extends AbstractBusClientServerTestBase {
 
     }
     
+    @Test
+    public void testCreateDispatchWithEPR() throws Exception {
+
+        URL wsdl = getClass().getResource("/wsdl/hello_world.wsdl");
+        assertNotNull(wsdl);
+
+        SOAPService service = new SOAPService(wsdl, SERVICE_NAME);
+        assertNotNull(service);
+
+        W3CEndpointReferenceBuilder builder = new  W3CEndpointReferenceBuilder();
+        builder.address("http://localhost:" 
+                        + greeterPort 
+                        + "/SOAPDispatchService/SoapDispatchPort");
+        builder.serviceName(SERVICE_NAME);
+        builder.endpointName(PORT_NAME);
+        W3CEndpointReference w3cEpr = builder.build();
+        Dispatch<SOAPMessage> disp = service
+            .createDispatch(w3cEpr, SOAPMessage.class, Service.Mode.MESSAGE);
+        InputStream is = getClass().getResourceAsStream("resources/GreetMeDocLiteralReq.xml");
+        SOAPMessage soapReqMsg = MessageFactory.newInstance().createMessage(null, is);
+        assertNotNull(soapReqMsg);
+        SOAPMessage soapResMsg = disp.invoke(soapReqMsg);
+        
+        assertNotNull(soapResMsg);
+        String expected = "Hello TestSOAPInputMessage";
+        assertEquals("Response should be : Hello TestSOAPInputMessage", expected, soapResMsg.getSOAPBody()
+            .getTextContent().trim());
+    }
+    
     @Test
     public void testDOMSourceMESSAGE() throws Exception {
         /*URL wsdl = getClass().getResource("/wsdl/hello_world.wsdl");
@@ -220,12 +318,14 @@ public class DispatchClientServerTest extends AbstractBusClientServerTestBase {
 
         SOAPService service = new SOAPService(wsdl, serviceName);
         assertNotNull(service);*/
-        Service service = Service.create(serviceName);
+        Service service = Service.create(SERVICE_NAME);
         assertNotNull(service);
-        service.addPort(portName, "http://schemas.xmlsoap.org/soap/", 
-                        "http://localhost:9006/SOAPDispatchService/SoapDispatchPort");
+        service.addPort(PORT_NAME, "http://schemas.xmlsoap.org/soap/", 
+                        "http://localhost:" 
+                        + greeterPort
+                        + "/SOAPDispatchService/SoapDispatchPort");
 
-        Dispatch<DOMSource> disp = service.createDispatch(portName, DOMSource.class, Service.Mode.MESSAGE);
+        Dispatch<DOMSource> disp = service.createDispatch(PORT_NAME, DOMSource.class, Service.Mode.MESSAGE);
 
         InputStream is = getClass().getResourceAsStream("resources/GreetMeDocLiteralReq.xml");
         SOAPMessage soapReqMsg = MessageFactory.newInstance().createMessage(null, is);
@@ -239,6 +339,12 @@ public class DispatchClientServerTest extends AbstractBusClientServerTestBase {
 
         assertEquals("Response should be : Hello TestSOAPInputMessage", expected, domResMsg.getNode()
             .getFirstChild().getTextContent().trim());
+        
+        Element el = (Element)domResMsg.getNode().getFirstChild();
+        System.out.println(DOMUtils.getElementQName(el));
+        assertEquals("gmns", el.lookupPrefix("http://apache.org/hello_world_soap_http/types"));
+        assertEquals("http://apache.org/hello_world_soap_http/types", 
+                     el.lookupNamespaceURI("gmns"));
 
         // Test invoke oneway
         InputStream is1 = getClass().getResourceAsStream("resources/GreetMeDocLiteralReq1.xml");
@@ -283,12 +389,13 @@ public class DispatchClientServerTest extends AbstractBusClientServerTestBase {
 
         SOAPService service = new SOAPService(wsdl, serviceName);
         assertNotNull(service);*/
-        Service service = Service.create(serviceName);
+        Service service = Service.create(SERVICE_NAME);
         assertNotNull(service);
-        service.addPort(portName, "http://schemas.xmlsoap.org/soap/", 
-                        "http://localhost:9006/SOAPDispatchService/SoapDispatchPort");
+        service.addPort(PORT_NAME, "http://schemas.xmlsoap.org/soap/", 
+                        "http://localhost:"
+                        + greeterPort + "/SOAPDispatchService/SoapDispatchPort");
 
-        Dispatch<DOMSource> disp = service.createDispatch(portName, DOMSource.class, Service.Mode.PAYLOAD);
+        Dispatch<DOMSource> disp = service.createDispatch(PORT_NAME, DOMSource.class, Service.Mode.PAYLOAD);
 
         InputStream is = getClass().getResourceAsStream("resources/GreetMeDocLiteralReq.xml");
         SOAPMessage soapReqMsg = MessageFactory.newInstance().createMessage(null, is);
@@ -356,11 +463,15 @@ public class DispatchClientServerTest extends AbstractBusClientServerTestBase {
         URL wsdl = getClass().getResource("/wsdl/hello_world.wsdl");
         assertNotNull(wsdl);
 
-        SOAPService service = new SOAPService(wsdl, serviceName);
+        SOAPService service = new SOAPService(wsdl, SERVICE_NAME);
         assertNotNull(service);
         
         JAXBContext jc = JAXBContext.newInstance("org.apache.hello_world_soap_http.types");
-        Dispatch<Object> disp = service.createDispatch(portName, jc, Service.Mode.PAYLOAD);
+        Dispatch<Object> disp = service.createDispatch(PORT_NAME, jc, Service.Mode.PAYLOAD);
+        disp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY,
+                                     "http://localhost:" 
+                                     + greeterPort
+                                     + "/SOAPDispatchService/SoapDispatchPort");
 
         String expected = "Hello Jeeves";
         GreetMe greetMe = new GreetMe();
@@ -444,14 +555,18 @@ public class DispatchClientServerTest extends AbstractBusClientServerTestBase {
         assertNotNull(wsdl);
 
         String bindingId = "http://schemas.xmlsoap.org/wsdl/soap/";
-        String endpointUrl = "http://localhost:9006/SOAPDispatchService/SoapDispatchPort";
+        String endpointUrl = "http://localhost:" + greeterPort + "/SOAPDispatchService/SoapDispatchPort";
         
-        Service service = Service.create(wsdl, serviceName);
-        service.addPort(portName, bindingId, endpointUrl);
+        Service service = Service.create(wsdl, SERVICE_NAME);
+        service.addPort(PORT_NAME, bindingId, endpointUrl);
         assertNotNull(service);
         
         JAXBContext jc = JAXBContext.newInstance("org.apache.hello_world_soap_http.types");
-        Dispatch<Object> disp = service.createDispatch(portName, jc, Service.Mode.PAYLOAD);
+        Dispatch<Object> disp = service.createDispatch(PORT_NAME, jc, Service.Mode.PAYLOAD);
+        disp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY,
+                                     "http://localhost:" 
+                                     + greeterPort
+                                     + "/SOAPDispatchService/SoapDispatchPort");
 
         String expected = "Hello Jeeves";
         GreetMe greetMe = new GreetMe();
@@ -480,7 +595,7 @@ public class DispatchClientServerTest extends AbstractBusClientServerTestBase {
         URL wsdl = getClass().getResource("/wsdl/hello_world.wsdl");
         assertNotNull(wsdl);
 
-        SOAPService service = new SOAPService(wsdl, serviceName);
+        SOAPService service = new SOAPService(wsdl, SERVICE_NAME);
         assertNotNull(service);
 
         InputStream is = getClass().getResourceAsStream("resources/GreetMeDocLiteralReq.xml");
@@ -488,7 +603,11 @@ public class DispatchClientServerTest extends AbstractBusClientServerTestBase {
         SAXSource saxSourceReq = new SAXSource(inputSource);
         assertNotNull(saxSourceReq);
 
-        Dispatch<SAXSource> disp = service.createDispatch(portName, SAXSource.class, Service.Mode.MESSAGE);
+        Dispatch<SAXSource> disp = service.createDispatch(PORT_NAME, SAXSource.class, Service.Mode.MESSAGE);
+        disp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY,
+                                     "http://localhost:" 
+                                     + greeterPort
+                                     + "/SOAPDispatchService/SoapDispatchPort");
         SAXSource saxSourceResp = disp.invoke(saxSourceReq);
         assertNotNull(saxSourceResp);
         String expected = "Hello TestSOAPInputMessage";
@@ -535,10 +654,14 @@ public class DispatchClientServerTest extends AbstractBusClientServerTestBase {
         URL wsdl = getClass().getResource("/wsdl/hello_world.wsdl");
         assertNotNull(wsdl);
 
-        SOAPService service = new SOAPService(wsdl, serviceName);
+        SOAPService service = new SOAPService(wsdl, SERVICE_NAME);
         assertNotNull(service);
 
-        Dispatch<SAXSource> disp = service.createDispatch(portName, SAXSource.class, Service.Mode.PAYLOAD);
+        Dispatch<SAXSource> disp = service.createDispatch(PORT_NAME, SAXSource.class, Service.Mode.PAYLOAD);
+        disp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY,
+                                     "http://localhost:" 
+                                     + greeterPort
+                                     + "/SOAPDispatchService/SoapDispatchPort");
 
         // Test request-response
         InputStream is = getClass().getResourceAsStream("resources/GreetMeDocLiteralSOAPBodyReq.xml");
@@ -602,10 +725,14 @@ public class DispatchClientServerTest extends AbstractBusClientServerTestBase {
         URL wsdl = getClass().getResource("/wsdl/hello_world.wsdl");
         assertNotNull(wsdl);
 
-        SOAPService service = new SOAPService(wsdl, serviceName);
+        SOAPService service = new SOAPService(wsdl, SERVICE_NAME);
         assertNotNull(service);
-        Dispatch<StreamSource> disp = service.createDispatch(portName, StreamSource.class,
+        Dispatch<StreamSource> disp = service.createDispatch(PORT_NAME, StreamSource.class,
                                                              Service.Mode.MESSAGE);
+        disp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY,
+                                     "http://localhost:" 
+                                     + greeterPort
+                                     + "/SOAPDispatchService/SoapDispatchPort");
 
         InputStream is = getClass().getResourceAsStream("resources/GreetMeDocLiteralReq.xml");
         StreamSource streamSourceReq = new StreamSource(is);
@@ -649,10 +776,14 @@ public class DispatchClientServerTest extends AbstractBusClientServerTestBase {
         URL wsdl = getClass().getResource("/wsdl/hello_world.wsdl");
         assertNotNull(wsdl);
 
-        SOAPService service = new SOAPService(wsdl, serviceName);
+        SOAPService service = new SOAPService(wsdl, SERVICE_NAME);
         assertNotNull(service);
-        Dispatch<StreamSource> disp = service.createDispatch(portName, StreamSource.class,
+        Dispatch<StreamSource> disp = service.createDispatch(PORT_NAME, StreamSource.class,
                                                              Service.Mode.PAYLOAD);
+        disp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY,
+                                     "http://localhost:" 
+                                     + greeterPort
+                                     + "/SOAPDispatchService/SoapDispatchPort");
 
         InputStream is = getClass().getResourceAsStream("resources/GreetMeDocLiteralSOAPBodyReq.xml");
         StreamSource streamSourceReq = new StreamSource(is);
@@ -737,6 +868,8 @@ public class DispatchClientServerTest extends AbstractBusClientServerTestBase {
 
         public void handleResponse(Response<Object> response) {
             try {
+                //response context shouldn't be empty even with fault response message
+                assertTrue(response.getContext().size() != 0);
                 reply = response.get();
             } catch (Exception e) {
                 //e.printStackTrace();
