@@ -19,14 +19,13 @@
 
 package org.apache.cxf.systest.ws.saml;
 
+import java.math.BigInteger;
 import java.net.URL;
 
 import javax.crypto.Cipher;
 import javax.crypto.SecretKey;
 import javax.crypto.spec.SecretKeySpec;
-import javax.xml.namespace.QName;
 import javax.xml.ws.BindingProvider;
-import javax.xml.ws.Service;
 
 import org.apache.cxf.Bus;
 import org.apache.cxf.bus.spring.SpringBusFactory;
@@ -35,19 +34,17 @@ import org.apache.cxf.systest.ws.saml.server.Server;
 import org.apache.cxf.testutil.common.AbstractBusClientServerTestBase;
 import org.apache.ws.security.saml.ext.builder.SAML2Constants;
 
-import org.example.contract.doubleit.DoubleItPortType;
-
 import org.junit.BeforeClass;
 
+import wssec.saml.DoubleItPortType;
+import wssec.saml.DoubleItService;
+
 /**
  * A set of tests for SAML Tokens.
  */
 public class SamlTokenTest extends AbstractBusClientServerTestBase {
     static final String PORT = allocatePort(Server.class);
     static final String PORT2 = allocatePort(Server.class, 2);
-    
-    private static final String NAMESPACE = "http://www.example.org/contract/DoubleIt";
-    private static final QName SERVICE_QNAME = new QName(NAMESPACE, "DoubleItService");
 
     private boolean unrestrictedPoliciesInstalled = checkUnrestrictedPoliciesInstalled();
     
@@ -71,15 +68,13 @@ public class SamlTokenTest extends AbstractBusClientServerTestBase {
         SpringBusFactory.setDefaultBus(bus);
         SpringBusFactory.setThreadDefaultBus(bus);
 
-        URL wsdl = SamlTokenTest.class.getResource("DoubleItSaml.wsdl");
-        Service service = Service.create(wsdl, SERVICE_QNAME);
-        QName portQName = new QName(NAMESPACE, "DoubleItSaml1TransportPort");
-        DoubleItPortType saml1Port = 
-                service.getPort(portQName, DoubleItPortType.class);
+        DoubleItService service = new DoubleItService();
+        
+        DoubleItPortType saml1Port = service.getDoubleItSaml1TransportPort();
         updateAddressPort(saml1Port, PORT2);
         
         try {
-            saml1Port.doubleIt(25);
+            saml1Port.doubleIt(BigInteger.valueOf(25));
             fail("Expected failure on an invocation with no SAML Assertion");
         } catch (javax.xml.ws.soap.SOAPFaultException ex) {
             assertTrue(ex.getMessage().contains("No SAML CallbackHandler available"));
@@ -89,7 +84,7 @@ public class SamlTokenTest extends AbstractBusClientServerTestBase {
             "ws-security.saml-callback-handler", new SamlCallbackHandler()
         );
         try {
-            saml1Port.doubleIt(25);
+            saml1Port.doubleIt(BigInteger.valueOf(25));
             fail("Expected failure on an invocation with a SAML2 Assertion");
         } catch (javax.xml.ws.soap.SOAPFaultException ex) {
             assertTrue(ex.getMessage().contains("Wrong SAML Version"));
@@ -98,8 +93,8 @@ public class SamlTokenTest extends AbstractBusClientServerTestBase {
         ((BindingProvider)saml1Port).getRequestContext().put(
             "ws-security.saml-callback-handler", new SamlCallbackHandler(false)
         );
-        int result = saml1Port.doubleIt(25);
-        assertTrue(result == 50);
+        BigInteger result = saml1Port.doubleIt(BigInteger.valueOf(25));
+        assertTrue(result.equals(BigInteger.valueOf(50)));
     }
     
     @org.junit.Test
@@ -115,15 +110,13 @@ public class SamlTokenTest extends AbstractBusClientServerTestBase {
         SpringBusFactory.setDefaultBus(bus);
         SpringBusFactory.setThreadDefaultBus(bus);
 
-        URL wsdl = SamlTokenTest.class.getResource("DoubleItSaml.wsdl");
-        Service service = Service.create(wsdl, SERVICE_QNAME);
-        QName portQName = new QName(NAMESPACE, "DoubleItSaml2SymmetricPort");
-        DoubleItPortType saml2Port = 
-                service.getPort(portQName, DoubleItPortType.class);
+        DoubleItService service = new DoubleItService();
+        
+        DoubleItPortType saml2Port = service.getDoubleItSaml2SymmetricPort();
         updateAddressPort(saml2Port, PORT);
        
         try {
-            saml2Port.doubleIt(25);
+            saml2Port.doubleIt(BigInteger.valueOf(25));
             fail("Expected failure on an invocation with no SAML Assertion");
         } catch (javax.xml.ws.soap.SOAPFaultException ex) {
             assertTrue(ex.getMessage().contains("No SAML CallbackHandler available"));
@@ -133,7 +126,7 @@ public class SamlTokenTest extends AbstractBusClientServerTestBase {
             "ws-security.saml-callback-handler", new SamlCallbackHandler(false)
         );
         try {
-            saml2Port.doubleIt(25);
+            saml2Port.doubleIt(BigInteger.valueOf(25));
             fail("Expected failure on an invocation with a SAML1 Assertion");
         } catch (javax.xml.ws.soap.SOAPFaultException ex) {
             assertTrue(ex.getMessage().contains("Wrong SAML Version"));
@@ -142,8 +135,8 @@ public class SamlTokenTest extends AbstractBusClientServerTestBase {
         ((BindingProvider)saml2Port).getRequestContext().put(
             "ws-security.saml-callback-handler", new SamlCallbackHandler()
         );
-        int result = saml2Port.doubleIt(25);
-        assertTrue(result == 50);
+        BigInteger result = saml2Port.doubleIt(BigInteger.valueOf(25));
+        assertTrue(result.equals(BigInteger.valueOf(50)));
         
         try {
             SamlCallbackHandler callbackHandler = 
@@ -152,7 +145,7 @@ public class SamlTokenTest extends AbstractBusClientServerTestBase {
             ((BindingProvider)saml2Port).getRequestContext().put(
                 "ws-security.saml-callback-handler", callbackHandler
             );
-            saml2Port.doubleIt(25);
+            saml2Port.doubleIt(BigInteger.valueOf(25));
             fail("Expected failure on an invocation with a invalid SAML2 Assertion");
         } catch (javax.xml.ws.soap.SOAPFaultException ex) {
             assertTrue(ex.getMessage().contains("SAML token security failure"));
@@ -177,11 +170,9 @@ public class SamlTokenTest extends AbstractBusClientServerTestBase {
         SpringBusFactory.setDefaultBus(bus);
         SpringBusFactory.setThreadDefaultBus(bus);
 
-        URL wsdl = SamlTokenTest.class.getResource("DoubleItSaml.wsdl");
-        Service service = Service.create(wsdl, SERVICE_QNAME);
-        QName portQName = new QName(NAMESPACE, "DoubleItSaml2SymmetricSupportingPort");
-        DoubleItPortType saml2Port = 
-                service.getPort(portQName, DoubleItPortType.class);
+        DoubleItService service = new DoubleItService();
+        
+        DoubleItPortType saml2Port = service.getDoubleItSaml2SymmetricSupportingPort();
         updateAddressPort(saml2Port, PORT);
 
         ((BindingProvider)saml2Port).getRequestContext().put(
@@ -189,7 +180,7 @@ public class SamlTokenTest extends AbstractBusClientServerTestBase {
         );
         
         try {
-            saml2Port.doubleIt(25);
+            saml2Port.doubleIt(BigInteger.valueOf(25));
             fail("Expected failure on an invocation with an unsigned SAML SV Assertion");
         } catch (javax.xml.ws.soap.SOAPFaultException ex) {
             assertTrue(ex.getMessage().contains("Assertion fails sender-vouches requirements"));
@@ -210,15 +201,13 @@ public class SamlTokenTest extends AbstractBusClientServerTestBase {
         SpringBusFactory.setDefaultBus(bus);
         SpringBusFactory.setThreadDefaultBus(bus);
 
-        URL wsdl = SamlTokenTest.class.getResource("DoubleItSaml.wsdl");
-        Service service = Service.create(wsdl, SERVICE_QNAME);
-        QName portQName = new QName(NAMESPACE, "DoubleItSaml2AsymmetricPort");
-        DoubleItPortType saml2Port = 
-                service.getPort(portQName, DoubleItPortType.class);
+        DoubleItService service = new DoubleItService();
+        
+        DoubleItPortType saml2Port = service.getDoubleItSaml2AsymmetricPort();
         updateAddressPort(saml2Port, PORT);
 
         try {
-            saml2Port.doubleIt(25);
+            saml2Port.doubleIt(BigInteger.valueOf(25));
             fail("Expected failure on an invocation with no SAML Assertion");
         } catch (javax.xml.ws.soap.SOAPFaultException ex) {
             assertTrue(ex.getMessage().contains("No SAML CallbackHandler available"));
@@ -228,7 +217,7 @@ public class SamlTokenTest extends AbstractBusClientServerTestBase {
             "ws-security.saml-callback-handler", new SamlCallbackHandler(false)
         );
         try {
-            saml2Port.doubleIt(25);
+            saml2Port.doubleIt(BigInteger.valueOf(25));
             fail("Expected failure on an invocation with a SAML1 Assertion");
         } catch (javax.xml.ws.soap.SOAPFaultException ex) {
             assertTrue(ex.getMessage().contains("Wrong SAML Version"));
@@ -237,8 +226,8 @@ public class SamlTokenTest extends AbstractBusClientServerTestBase {
         ((BindingProvider)saml2Port).getRequestContext().put(
             "ws-security.saml-callback-handler", new SamlCallbackHandler()
         );
-        int result = saml2Port.doubleIt(25);
-        assertTrue(result == 50);
+        BigInteger result = saml2Port.doubleIt(BigInteger.valueOf(25));
+        assertTrue(result.equals(BigInteger.valueOf(50)));
     }
     
     @org.junit.Test
@@ -251,22 +240,21 @@ public class SamlTokenTest extends AbstractBusClientServerTestBase {
         SpringBusFactory.setDefaultBus(bus);
         SpringBusFactory.setThreadDefaultBus(bus);
 
-        URL wsdl = SamlTokenTest.class.getResource("DoubleItSaml.wsdl");
-        Service service = Service.create(wsdl, SERVICE_QNAME);
-        QName portQName = new QName(NAMESPACE, "DoubleItSaml1SelfSignedTransportPort");
-        DoubleItPortType saml1Port = 
-                service.getPort(portQName, DoubleItPortType.class);
+        DoubleItService service = new DoubleItService();
+        
+        DoubleItPortType saml1Port = service.getDoubleItSaml1SelfSignedTransportPort();
         updateAddressPort(saml1Port, PORT2);
         
         ((BindingProvider)saml1Port).getRequestContext().put(
             "ws-security.saml-callback-handler", new SamlCallbackHandler(false)
         );
-        int result = saml1Port.doubleIt(25);
-        assertTrue(result == 50);
+        BigInteger result = saml1Port.doubleIt(BigInteger.valueOf(25));
+        assertTrue(result.equals(BigInteger.valueOf(50)));
     }
     
     @org.junit.Test
-    public void testAsymmetricSamlInitiator() throws Exception {
+    @org.junit.Ignore
+    public void testSaml2OverSymmetricProtection() throws Exception {
 
         if (!unrestrictedPoliciesInstalled) {
             return;
@@ -278,20 +266,16 @@ public class SamlTokenTest extends AbstractBusClientServerTestBase {
         SpringBusFactory.setDefaultBus(bus);
         SpringBusFactory.setThreadDefaultBus(bus);
 
-        URL wsdl = SamlTokenTest.class.getResource("DoubleItSaml.wsdl");
-        Service service = Service.create(wsdl, SERVICE_QNAME);
-        QName portQName = new QName(NAMESPACE, "DoubleItAsymmetricSamlInitiatorPort");
-        DoubleItPortType saml2Port = 
-                service.getPort(portQName, DoubleItPortType.class);
+        DoubleItService service = new DoubleItService();
+        
+        DoubleItPortType saml2Port = service.getDoubleItSaml2SymmetricProtectionPort();
         updateAddressPort(saml2Port, PORT);
         
-        SamlCallbackHandler callbackHandler = new SamlCallbackHandler();
-        callbackHandler.setConfirmationMethod(SAML2Constants.CONF_HOLDER_KEY);
         ((BindingProvider)saml2Port).getRequestContext().put(
-            "ws-security.saml-callback-handler", callbackHandler
+            "ws-security.saml-callback-handler", new SamlCallbackHandler()
         );
-        int result = saml2Port.doubleIt(25);
-        assertTrue(result == 50);
+        BigInteger result = saml2Port.doubleIt(BigInteger.valueOf(25));
+        assertTrue(result.equals(BigInteger.valueOf(50)));
     }
     
     @org.junit.Test
@@ -307,21 +291,20 @@ public class SamlTokenTest extends AbstractBusClientServerTestBase {
         SpringBusFactory.setDefaultBus(bus);
         SpringBusFactory.setThreadDefaultBus(bus);
 
-        URL wsdl = SamlTokenTest.class.getResource("DoubleItSaml.wsdl");
-        Service service = Service.create(wsdl, SERVICE_QNAME);
-        QName portQName = new QName(NAMESPACE, "DoubleItSaml2SymmetricSignedElementsPort");
-        DoubleItPortType saml2Port = 
-                service.getPort(portQName, DoubleItPortType.class);
+        DoubleItService service = new DoubleItService();
+        
+        DoubleItPortType saml2Port = service.getDoubleItSaml2SymmetricSignedElementsPort();
         updateAddressPort(saml2Port, PORT);
         
         ((BindingProvider)saml2Port).getRequestContext().put(
             "ws-security.saml-callback-handler", new SamlCallbackHandler()
         );
-        int result = saml2Port.doubleIt(25);
-        assertTrue(result == 50);
+        BigInteger result = saml2Port.doubleIt(BigInteger.valueOf(25));
+        assertTrue(result.equals(BigInteger.valueOf(50)));
     }
     
     @org.junit.Test
+    @org.junit.Ignore
     public void testSaml2EndorsingOverTransport() throws Exception {
 
         SpringBusFactory bf = new SpringBusFactory();
@@ -331,134 +314,19 @@ public class SamlTokenTest extends AbstractBusClientServerTestBase {
         SpringBusFactory.setDefaultBus(bus);
         SpringBusFactory.setThreadDefaultBus(bus);
 
-        URL wsdl = SamlTokenTest.class.getResource("DoubleItSaml.wsdl");
-        Service service = Service.create(wsdl, SERVICE_QNAME);
-        QName portQName = new QName(NAMESPACE, "DoubleItSaml2EndorsingTransportPort");
-        DoubleItPortType saml2Port = 
-                service.getPort(portQName, DoubleItPortType.class);
-        updateAddressPort(saml2Port, PORT2);
+        DoubleItService service = new DoubleItService();
         
-        SamlCallbackHandler callbackHandler = new SamlCallbackHandler();
-        callbackHandler.setConfirmationMethod(SAML2Constants.CONF_HOLDER_KEY);
-        ((BindingProvider)saml2Port).getRequestContext().put(
-            "ws-security.saml-callback-handler", callbackHandler
-        );
-
-        int result = saml2Port.doubleIt(25);
-        assertTrue(result == 50);
-    }
-
-    @org.junit.Test
-    public void testSaml2OverAsymmetricSignedEncrypted() throws Exception {
-
-        if (!unrestrictedPoliciesInstalled) {
-            return;
-        }
+        DoubleItPortType saml1Port = service.getDoubleItSaml2EndorsingTransportPort();
+        updateAddressPort(saml1Port, PORT2);
         
-        SpringBusFactory bf = new SpringBusFactory();
-        URL busFile = SamlTokenTest.class.getResource("client/client.xml");
-
-        Bus bus = bf.createBus(busFile.toString());
-        SpringBusFactory.setDefaultBus(bus);
-        SpringBusFactory.setThreadDefaultBus(bus);
-
-        URL wsdl = SamlTokenTest.class.getResource("DoubleItSaml.wsdl");
-        Service service = Service.create(wsdl, SERVICE_QNAME);
-        QName portQName = new QName(NAMESPACE, "DoubleItSaml2AsymmetricSignedEncryptedPort");
-        DoubleItPortType saml2Port = 
-                service.getPort(portQName, DoubleItPortType.class);
-        updateAddressPort(saml2Port, PORT);
-        
-        ((BindingProvider)saml2Port).getRequestContext().put(
+        ((BindingProvider)saml1Port).getRequestContext().put(
             "ws-security.saml-callback-handler", new SamlCallbackHandler()
         );
-        int result = saml2Port.doubleIt(25);
-        assertTrue(result == 50);
+
+        BigInteger result = saml1Port.doubleIt(BigInteger.valueOf(25));
+        assertTrue(result.equals(BigInteger.valueOf(50)));
     }
     
-    @org.junit.Test
-    public void testSaml2OverAsymmetricEncrypted() throws Exception {
-
-        if (!unrestrictedPoliciesInstalled) {
-            return;
-        }
-        
-        SpringBusFactory bf = new SpringBusFactory();
-        URL busFile = SamlTokenTest.class.getResource("client/client.xml");
-
-        Bus bus = bf.createBus(busFile.toString());
-        SpringBusFactory.setDefaultBus(bus);
-        SpringBusFactory.setThreadDefaultBus(bus);
-
-        URL wsdl = SamlTokenTest.class.getResource("DoubleItSaml.wsdl");
-        Service service = Service.create(wsdl, SERVICE_QNAME);
-        QName portQName = new QName(NAMESPACE, "DoubleItSaml2AsymmetricEncryptedPort");
-        DoubleItPortType saml2Port = 
-                service.getPort(portQName, DoubleItPortType.class);
-        updateAddressPort(saml2Port, PORT);
-        
-        SamlCallbackHandler callbackHandler = new SamlCallbackHandler();
-        callbackHandler.setConfirmationMethod(SAML2Constants.CONF_BEARER);
-        ((BindingProvider)saml2Port).getRequestContext().put(
-            "ws-security.saml-callback-handler", callbackHandler
-        );
-        int result = saml2Port.doubleIt(25);
-        assertTrue(result == 50);
-    }
-    
-    
-    @org.junit.Test
-    public void testSaml2EndorsingEncryptedOverTransport() throws Exception {
-
-        SpringBusFactory bf = new SpringBusFactory();
-        URL busFile = SamlTokenTest.class.getResource("client/client.xml");
-
-        Bus bus = bf.createBus(busFile.toString());
-        SpringBusFactory.setDefaultBus(bus);
-        SpringBusFactory.setThreadDefaultBus(bus);
-
-        URL wsdl = SamlTokenTest.class.getResource("DoubleItSaml.wsdl");
-        Service service = Service.create(wsdl, SERVICE_QNAME);
-        QName portQName = new QName(NAMESPACE, "DoubleItSaml2EndorsingEncryptedTransportPort");
-        DoubleItPortType saml2Port = 
-                service.getPort(portQName, DoubleItPortType.class);
-        updateAddressPort(saml2Port, PORT2);
-        
-        SamlCallbackHandler callbackHandler = new SamlCallbackHandler();
-        callbackHandler.setConfirmationMethod(SAML2Constants.CONF_HOLDER_KEY);
-        ((BindingProvider)saml2Port).getRequestContext().put(
-            "ws-security.saml-callback-handler", callbackHandler
-        );
-
-        int result = saml2Port.doubleIt(25);
-        assertTrue(result == 50);
-    }
-    
-    @org.junit.Test
-    public void testNoSamlToken() throws Exception {
-
-        SpringBusFactory bf = new SpringBusFactory();
-        URL busFile = SamlTokenTest.class.getResource("client/client.xml");
-
-        Bus bus = bf.createBus(busFile.toString());
-        SpringBusFactory.setDefaultBus(bus);
-        SpringBusFactory.setThreadDefaultBus(bus);
-
-        URL wsdl = SamlTokenTest.class.getResource("DoubleItSaml.wsdl");
-        Service service = Service.create(wsdl, SERVICE_QNAME);
-        QName portQName = new QName(NAMESPACE, "DoubleItInlinePolicyPort");
-        DoubleItPortType saml2Port = 
-                service.getPort(portQName, DoubleItPortType.class);
-        updateAddressPort(saml2Port, PORT2);
-        
-        try {
-            saml2Port.doubleIt(25);
-            fail("Failure expected on no SamlToken");
-        } catch (javax.xml.ws.soap.SOAPFaultException ex) {
-            String error = "The received token does not match the token inclusion requirement";
-            assertTrue(ex.getMessage().contains(error));
-        }
-    }
     
     
     private boolean checkUnrestrictedPoliciesInstalled() {
